package delta;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;




public class DeltaApp extends JFrame implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static final String DEFAULT_TEXT_FILE_NAME = "text.xml";
	
	public static final String WHITE_SPACE = " :;?.,\n\r\t";
	public static final int DEFAULT_TRAINING_SECTION_SIZE = 1000;
	public static final int DEFAULT_NUM_MFW = 150;
	private TreeMap <String,Integer> orderedLexicon;
	private TreeMap <String,Integer> orderedLexicon2;
	private TreeMap <String,Vector<Integer>> trainingLexicon;
	private TreeMap <String,Vector<Integer>> targetLexicon;
	// TreeMap <String,TokenFrequency> comparLexicon;
	private TrainingData trainingData;
	private TargetData authorData;
	private TargetData targetData;
	private JButton collectTokensBtn; 
	private JButton saveTypesBtn; 
	private JButton compareTypesBtn; 
	private JButton saveComparisionBtn; 
	private JButton buildTrigramBtn; 
	private JButton searchTrigramBtn; 
	private JButton collectTrainingData; 
	private JButton collectAuthorData; 
	private JButton collectTargetData; 
	private JButton collectMFW; 
	private JCheckBox useMFW; 
	private JCheckBox normalizeArabic; 
	private int tokenCount; 
	private int typeCount;
	private int sectionTokenCount; 


	private JTextField inputFld;
	private JTextField numMfwFld;
	private JTextField sizeOfTrainingSectionsFld;
	private JTextField trainingSectionSizeFld;
	private JTextArea textArea;	
	private JTextArea resultArea;	

	public DeltaApp(){
		this.setTitle("Delta");
		collectTokensBtn = new JButton("collect");
		collectTokensBtn.addActionListener(this); 
		collectTokensBtn.setActionCommand("c");
		
		buildTrigramBtn = new JButton("Build Trigrams");
		buildTrigramBtn.addActionListener(this); 
		buildTrigramBtn.setActionCommand("bt");
		
		searchTrigramBtn = new JButton("Search Trigrams");
		searchTrigramBtn.addActionListener(this); 
		searchTrigramBtn.setActionCommand("st");
		
		saveTypesBtn = new JButton("save");
		saveTypesBtn.addActionListener(this); 
		saveTypesBtn.setActionCommand("s");

		compareTypesBtn = new JButton("collect second");
		compareTypesBtn.addActionListener(this); 
		compareTypesBtn.setActionCommand("2");

		saveComparisionBtn = new JButton("save comparision");
		saveComparisionBtn.addActionListener(this); 
		saveComparisionBtn.setActionCommand("b");
		
		collectTrainingData = new JButton("collect training data");
		collectTrainingData.addActionListener(this); 
		collectTrainingData.setActionCommand("td");
		
		collectAuthorData = new JButton("collect author data");
		collectAuthorData.addActionListener(this); 
		collectAuthorData.setActionCommand("aud");

		collectTargetData = new JButton("collect target data");
		collectTargetData.addActionListener(this); 
		collectTargetData.setActionCommand("tgd");

		
		collectMFW = new JButton("collect MFW");
		collectMFW.addActionListener(this); 
		collectMFW.setActionCommand("mfw");

		
		
		inputFld = new JTextField(); 
		inputFld.setText("xxx_yyy");
		inputFld.setColumns(30);
		inputFld.setBorder(BorderFactory.createTitledBorder("Input"));
		numMfwFld = new JTextField(); 
		numMfwFld.setText("150");
		numMfwFld.setColumns(6);
		numMfwFld.setBorder(BorderFactory.createTitledBorder("Num MFW"));
		sizeOfTrainingSectionsFld = new JTextField(); 
		sizeOfTrainingSectionsFld.setText("50");
		sizeOfTrainingSectionsFld.setColumns(6);
		sizeOfTrainingSectionsFld.setBorder(BorderFactory.createTitledBorder("Size Tr.Sect."));
		textArea = new JTextArea();
		textArea.setBorder(BorderFactory.createTitledBorder("Text"));
		textArea.setText("xxx yyy zzz zzz xxx yyy zzz");
		resultArea = new JTextArea();
		resultArea.setBorder(BorderFactory.createTitledBorder("Results"));
		resultArea.setText("Results");
		useMFW = new JCheckBox("Use MFW");
		normalizeArabic = new JCheckBox("Normalize Arabic");
		
		JPanel fldPn = new JPanel(); 
		JPanel prmPn = new JPanel(); 
		JPanel btnPn = new JPanel(); 
		fldPn.setLayout(new GridLayout(2,1));
		btnPn.setLayout(new GridLayout(1,11));
		JPanel allPn = new JPanel(); 
		allPn.setLayout(new BorderLayout());
		btnPn.add(collectTokensBtn);
		btnPn.add(collectTrainingData);
		btnPn.add(collectAuthorData);
		btnPn.add(collectTargetData);
		btnPn.add(collectMFW);
		btnPn.add(saveTypesBtn);
		btnPn.add(compareTypesBtn);
		btnPn.add(saveComparisionBtn);
		btnPn.add(buildTrigramBtn);
		btnPn.add(searchTrigramBtn);
		fldPn.add(textArea);
		fldPn.add(resultArea);
		allPn.add(btnPn, BorderLayout.SOUTH);
		allPn.add(fldPn, BorderLayout.CENTER);
		prmPn.add(inputFld);
		prmPn.add(numMfwFld);
		prmPn.add(sizeOfTrainingSectionsFld);
		prmPn.add(useMFW);
		prmPn.add(normalizeArabic);
		allPn.add(prmPn, BorderLayout.NORTH);
		this.getContentPane().add(allPn);
	}
	
	public static void main(String[] args) {
		JFrame frm = new DeltaApp();
		frm.setBounds(50, 60, 1000, 600);
		frm.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if ("c".equals(e.getActionCommand())) {
	        collectTypes();
		}else if ("td".equals(e.getActionCommand())) {
			collectTrainingData3();
		}else if ("aud".equals(e.getActionCommand())) {
			 collectAuthorData();
		}else if ("tgd".equals(e.getActionCommand())) {
			 collectTargetData();
		}else if ("mfw".equals(e.getActionCommand())) {
			 collectMFW();
		}else if ("s".equals(e.getActionCommand())) {
			saveTokenCount();
		}else if ("2".equals(e.getActionCommand())) {
			openSecond();
		}else if ("b".equals(e.getActionCommand())) {
			saveSecondCount();
		}else if ("bt".equals(e.getActionCommand())) {
			buildTrigrams();	
		}else if ("st".equals(e.getActionCommand())) {
			searchTrigrams();
		}		
	}

	public void collectTypes(){
		TreeMap<String,Integer> lexicon = new   TreeMap<String,Integer>();
		
		loadText(lexicon);
		orderedLexicon = lexicon;
	}
	public void collectTrainingData(){
		TreeMap <String,Vector<Integer>> lexicon = new TreeMap <String,Vector<Integer>>();
		
		loadTrainingText(lexicon);
		//orderedLexicon = lexicon;
	}
	public void collectTrainingData2(){
		//TreeMap <String,Vector<Integer>> lexicon = new TreeMap <String,Vector<Integer>>();
		
		loadTrainingText2(trainingData);
		//orderedLexicon = lexicon;
	}
	public void collectTrainingData3(){
		loadTrainingText3(trainingData);
	}
	public void saveTokenCount(){
		saveTypes(orderedLexicon);
	}
	
	public void openSecond(){
		TreeMap<String,Integer> lexicon = new   TreeMap<String,Integer>();	
		loadText(lexicon);
		orderedLexicon2 = lexicon;	
	}
	public void saveSecondCount(){
		//saveTypes(orderedLexicon2);
		saveComparision(orderedLexicon, orderedLexicon2);
	}


	public void loadText(TreeMap<String,Integer> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		if(lexicon==null){
			lexicon = new   TreeMap<String,Integer>();
		}
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		//String path_std = path_prefix+"/text.txt";		
		 JFileChooser fc = new JFileChooser(new File(path_prefix));
		 fc.showOpenDialog(this);
		 File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			processLine(lineCnts, lexicon);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				processLine(lineCnts, lexicon);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		inputFld.setText("Ratio: "+tokenCount+"/"+typeCount+" = "+((double) tokenCount/typeCount));
	}


	public void processLine(String ln, TreeMap<String,Integer> lexicon){
		
		if(ln==null){
			return;
		}
		StringTokenizer tokenPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(tokenPrsr.hasMoreTokens()){
			String tkn = tokenPrsr.nextToken();
			Integer cnt = lexicon.get(tkn);
			if(cnt!=null){
				lexicon.remove(tkn);
				lexicon.put(tkn, new Integer(cnt+1));
				tokenCount++;
			}else{
				lexicon.put(tkn, new Integer(1));
				tokenCount++;
				typeCount++;
			}
		}
	}
	public void loadTrainingText(TreeMap<String,Vector<Integer>> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		sectionTokenCount = 0; 
		Vector<Integer> sectionTokenSizes = new Vector<Integer>();
		int trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		int sectionNum = 1;
		//if(lexicon==null){
			lexicon = new   TreeMap<String,Vector<Integer>>();
		//}
		BufferedReader bReader = null;		
		String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_home_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			sectionTokenCount = processLineTrainingData(lineCnts, lexicon, sectionTokenCount, trainingSectionSize,sectionTokenSizes);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				sectionTokenCount = processLineTrainingData(lineCnts, lexicon, sectionTokenCount, trainingSectionSize, sectionTokenSizes);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		sectionTokenSizes.add(new Integer(sectionTokenCount));
		
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		
		//List<String> frqList = new  LinkedList();
		TreeMap<Double,Vector<String>>frqList = new   TreeMap<Double,Vector<String>>();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String type = it.next();
			Vector<Integer> cnt = lexicon.get(type);
			double mean = getMeanFrequency(cnt, sectionTokenSizes);
			double stdev = getStdev(cnt, mean, sectionTokenSizes);
			String entryStr = type+"\t"+mean+"\t"+stdev ;
			Double frq = new Double(-mean);
			Vector<String> entry = frqList.get(frq);
			if(entry==null){
				Vector<String> newEntry = new Vector<String>();
				newEntry.add(entryStr);
				frqList.put(frq, newEntry);
			}else{
				entry.add(entryStr);
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Double(mean),type+"\t"+mean+"\t"+stdev );
		}
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		StringBuffer outStr = new StringBuffer();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<String> vls = frqList.get(key);
			for(int i=0; i< vls.size(); i++){
				outStr.append(vls.get(i)+"\n");
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Integer(mean),type+"\t"+mean+"\t"+stdev )
		}
	
		

		resultArea.setText(outStr.toString());
	}

	public int processLineTrainingData(String ln, TreeMap<String,Vector<Integer>> lexicon, int sectionTokenCount, int trainingSectionSize, Vector<Integer> sectionTokenSizes){
		if(ln==null){
			return sectionTokenCount;
		}
		int sectionNum = sectionTokenSizes.size()+1;
		StringTokenizer tokenPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(tokenPrsr.hasMoreTokens()){
			sectionTokenCount++;
			if(sectionTokenCount>trainingSectionSize){	// start new section
				sectionNum++;
				sectionTokenCount=1;
				sectionTokenSizes.add(new Integer(trainingSectionSize));
				sectionNum = sectionTokenSizes.size();
			}
			String tkn = tokenPrsr.nextToken();
			Vector<Integer> cnt = lexicon.get(tkn);
			if(cnt==null){	// new type
				cnt = new Vector<Integer>();
			}
			if(cnt.size()<sectionNum){	// no counter for this section
				for(int i=cnt.size(); i<sectionNum-1; i++){	// missing counters in between
					cnt.add(new Integer(0));
				}
				cnt.add(new Integer(1)); // counter for this section
			}else{						// counter for this section exists
				Integer newCnt = new Integer(cnt.get(sectionNum-1)+1);
				cnt.set(sectionNum-1, newCnt);
				//cntr++;
			}
			lexicon.put(tkn, cnt);
			tokenCount++;
		}
		return sectionTokenCount; 
	}

	public double getMeanFrequency(Vector<Integer> cnt, Vector<Integer>  sectionTokenSizes){
		double count = 0.0;
		double allToken = 0.0;
		int sz = cnt.size();
		int sctSz = sectionTokenSizes.size();
		for(int i=0; i<sz; i++){
			count += cnt.get(i);
		}
		for(int i=0; i<sctSz; i++){
			allToken += (double)sectionTokenSizes.get(i);
		}
		double mean = count/allToken;
		return mean;
	}
	public double getStdev(Vector<Integer> cnt, double mean,Vector<Integer>  sectionTokenSizes){
		double stdev = 0.0;
		int sz = cnt.size();
		for(int i=0; i<sz; i++){
			double invNum = 1.0/(double)sectionTokenSizes.get(i);
			double tmp = mean- invNum*cnt.get(i);
			stdev += tmp*tmp;
		}
		return Math.sqrt(stdev/(sz-1));
	}
	
	
	public void loadTargetText(TreeMap<String,Vector<Integer>> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		lexicon = new   TreeMap<String,Vector<Integer>>();
		BufferedReader bReader = null;		
		String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_home_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			processLineTargetData(lineCnts, lexicon);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				processLineTargetData(lineCnts, lexicon);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//sectionTokenSizes.add(new Integer(sectionTokenCount));
		
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		
		//List<String> frqList = new  LinkedList();
		TreeMap<Double,Vector<String>>frqList = new   TreeMap<Double,Vector<String>>();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String type = it.next();
			Vector<Integer> cnt = lexicon.get(type);
			//double mean = getMeanFrequency(cnt, sectionTokenSizes);
			//double stdev = getStdev(cnt, mean, sectionTokenSizes);
			//String entryStr = type+"\t"+mean+"\t"+stdev ;
			//Double frq = new Double(-mean);
			//Vector<String> entry = frqList.get(frq);
			//if(entry==null){
				Vector<String> newEntry = new Vector<String>();
			//	newEntry.add(entryStr);
			//	frqList.put(frq, newEntry);
			//}else{
			//	entry.add(entryStr);
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Double(mean),type+"\t"+mean+"\t"+stdev );
		}
		/*
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		StringBuffer outStr = new StringBuffer();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<String> vls = frqList.get(key);
			for(int i=0; i< vls.size(); i++){
				outStr.append(vls.get(i)+"\n");
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Integer(mean),type+"\t"+mean+"\t"+stdev )
		}
	
		

		resultArea.setText(outStr.toString());
		*/
	

	public void processLineTargetData(String ln, TreeMap<String,Vector<Integer>> lexicon){
		
	}
	public void loadTrainingText2(TrainingData tData){
		Vector<Integer> sectionTokenSizes = new Vector<Integer>();
		int trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		tData = new TrainingData(trainingSectionSize);
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			tData.addText(lineCnts);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				tData.addText(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		tData.calculateStatistics();

		resultArea.setText(tData.toVerboseString());
		trainingData = tData;
	}
	public void loadTrainingText3(TrainingData tData){
		Vector<Integer> sectionTokenSizes = new Vector<Integer>();
		int trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		tData = new TrainingData(trainingSectionSize);
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		resultArea.setText("Number of tokens = "+Integer.toString(cnt));
		/*
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			tData.addText(lineCnts);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				tData.addText(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		tData.calculateStatistics();

		resultArea.setText(tData.toVerboseString());
		trainingData = tData;
		*/
	}
	public int countLineTokens(String ln){
		if(ln==null){
			return 0;
		}
		StringTokenizer tknPrsr = new StringTokenizer(ln, WHITE_SPACE);
		return tknPrsr.countTokens();
	}
	public int countTokens(File selFile){
		JFileChooser fc = null;
		BufferedReader bReader = null;		

		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		int cnt = 0;
		String lineCnts = null;
		String ln = "";
		try {
			lineCnts = bReader.readLine();
			cnt= countLineTokens(lineCnts);
			
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				cnt += countLineTokens(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return cnt;
	}
	public void collectAuthorData(){
		authorData = new TargetData(trainingData);
		loadTargetText(trainingData, authorData);
	}
	public void collectTargetData(){
		targetData = new TargetData(trainingData);
		loadTargetText(trainingData, targetData);
		calculateDelta(authorData, targetData);
	}
	public void loadTargetText(TrainingData trnngData, TargetData tgData){
		//tgData = new TargetData(trnngData);
		BufferedReader bReader = null;		
		String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_home_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			tgData.addText(lineCnts);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				tgData.addText(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		tgData.calculateStatistics();

		resultArea.setText(tgData.toString());
		//trgtData = tgData;
	}
	public void calculateDelta(TargetData authorData, TargetData targetData){
		if(useMFW.isSelected()){
			targetData.compareMFW(authorData, textArea.getText());
			resultArea.setText(targetData.mfwToString());
		}else{
			targetData.compare(authorData);
			resultArea.setText(targetData.toString());
		}
		
	}
	public void collectMFW(){
		int numOfMFW = DEFAULT_NUM_MFW;
		try{
			String numOfMFWstr = numMfwFld.getText();
			numOfMFW = Integer.parseInt(numOfMFWstr);
		}catch(NumberFormatException e){
			
		}
		textArea.setText(trainingData.selectMFW(numOfMFW));
	}
	public static String normalizeArabic(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(!Character.isLetter(ch) || ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else if(ch=='\u0629'){
				outStr.append('\u0647');
			}else if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();

	}

	// ========================================================== OLD PART =============================================
	
	public void saveTypes(TreeMap<String,Integer> lexicon){
		if(lexicon==null){
			return;
		}
		Writer pWriter;
		String path_home_prefix = System.getProperty("user.home");		
		 JFileChooser fc = new JFileChooser(new File(path_home_prefix));		 
		int rv = fc.showSaveDialog(DeltaApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
		        new FileOutputStream(selFile.getPath()), "UTF8"));
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		while(it.hasNext()){
			String type = it.next();
			Integer cnt = lexicon.get(type);
			pWriter.write(type+"\t"+cnt.toString()+"\n");
		}
		pWriter.close();
		} catch (UnsupportedEncodingException e) {
		} catch (IOException e) {
		}
	}
	public void saveComparision(TreeMap<String,Integer> lex1, TreeMap<String,Integer> lex2){
		if(lex1==null || lex2 == null){
			return;
		}
		Writer pWriter;
		String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_home_prefix));		 
		int rv = fc.showSaveDialog(DeltaApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		if(selFile==null){
			 return;
		}
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
		        new FileOutputStream(selFile.getPath()), "UTF8"));
		List<String> lxList1 = new  LinkedList(lex1.keySet());
		Collections.sort(lxList1);
		Iterator<String> it1 = lxList1.iterator();
		while(it1.hasNext()){
			String type = it1.next();
			Integer cnt1 = lex1.get(type);
			Integer cnt2 = lex2.get(type);
			if(cnt2==null){
				pWriter.write(type+"\t"+cnt1.toString()+"\t0\n");
			}else{
				pWriter.write(type+"\t"+cnt1.toString()+"\t"+cnt2.toString()+"\n");
			}
		}

		List<String> lxList2 = new  LinkedList(lex2.keySet());
		Collections.sort(lxList2);
		Iterator<String> it2 = lxList2.iterator();
		while(it2.hasNext()){
			String type = it2.next();
			Integer cnt1 = lex1.get(type);
			Integer cnt2 = lex2.get(type);
			if(cnt1==null){
				pWriter.write(type+"\t0\t"+cnt2.toString()+"\n");
			}
		}

		
		pWriter.close();
		} catch (UnsupportedEncodingException e) {
		} catch (IOException e) {
		}
	}
	public void errorMessage(){
		resultArea.setText("An error occured.");
	}
	private TreeMap <String,Trigram> trigrams;
	private TreeMap <String,Trigram> contexts;
	private TreeMap <String,Trigram> cores;
	
	public boolean isKnownTrigram(String tgrmStr){
		return trigrams.containsKey(tgrmStr);
	}
	
	public void storeTrigram(Trigram trgrm){
		trigrams.put(trgrm.toString(), trgrm);
		//contexts.put(trgrm.getPrefix()+Trigram.SEPARATOR+trgrm.getSuffix(), trgrm);
		//.put(trgrm.getCore(), trgrm);
	}
	public void appendTrigram(Trigram trgrm){
		trigrams.put(trgrm.toString(), trgrm);
		//contexts.put(trgrm.getPrefix()+Trigram.SEPARATOR+trgrm.getSuffix(), trgrm);
		//cores.put(trgrm.getCore(), trgrm);
	}
	
	public void buildTrigrams(){
		trigrams = new   TreeMap<String,Trigram>();
		contexts = new   TreeMap<String,Trigram>();
		cores = new   TreeMap<String,Trigram>();
		String txtStr = textArea.getText();
		StringTokenizer wrdPrsr = new StringTokenizer(txtStr, WHITE_SPACE);
		//String [] txtPrts = txtStr.split("_");
		String prefStr = "_";
		if(wrdPrsr.hasMoreTokens()){
			prefStr =  wrdPrsr.nextToken();
		}
		String coreStr = "_";
		if(wrdPrsr.hasMoreTokens()){
			coreStr =  wrdPrsr.nextToken();
		}
		String suffStr = "_";
		while(wrdPrsr.hasMoreTokens()){
			suffStr =  wrdPrsr.nextToken();
			String tmpTrgrmStr = prefStr+Trigram.SEPARATOR+coreStr+Trigram.SEPARATOR+suffStr;
			if(!isKnownTrigram(tmpTrgrmStr)){
				storeTrigram(new Trigram(tmpTrgrmStr));
			}
			prefStr = coreStr;
			coreStr = suffStr;
		}
		List<String> lxList = new  LinkedList(trigrams.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		StringBuffer outStr = new StringBuffer();
		while(it.hasNext()){
			String keyStr = it.next();
			//Trigram trgrm = cores.get(keyStr);
			//outStr.append(keyStr+"\t"+trgrm.toString()+"\n");
			outStr.append(keyStr+"\n");
		}
		
		
		resultArea.setText(outStr.toString());
	}


public void searchTrigrams(){
		String trgrmStr = inputFld.getText();
		String [] trgrmPrts = trgrmStr.split("_");
		if(trgrmPrts.length!=2){
			errorMessage();
			return;
		}
		String textStr = textArea.getText();
		StringTokenizer lnPrsr = new StringTokenizer(textStr, "\n\r");
		
		String prefStr = trgrmPrts[0].trim();
		String suffStr = trgrmPrts[1].trim();
		
		
		resultArea.setText(prefStr+"\n"+suffStr);
	}
}
