package delta;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

public class DeltaProcedure {
	
	private TrainingLexicon mainSet;
	private TargetLexicon unknownAuthor;
	private ArrayList<TargetLexicon> knownAuthors; 
	private ArrayList<String> authorNames; 
	private ArrayList<Double> deltaValues; 
	private ArrayList<Double> cosineSimilarities; 
	
	public DeltaProcedure(TrainingLexicon mSet){
		mainSet = mSet;
		knownAuthors = new ArrayList<TargetLexicon>();
		authorNames = new ArrayList<String>();
	}
	public void setUnknownAuthor(TargetLexicon target){
		unknownAuthor = target;
	}
	
	public void addKnownAuthor(TargetLexicon target, String aName){
		knownAuthors.add(target);
		authorNames.add(aName);
	}
	
	public void calculateDelta(){
		TreeMap <String, Double> means = mainSet.getMeans();
		TreeMap <String, Double> stdevs = mainSet.getStdevs();
		unknownAuthor.calculateScores();
		unknownAuthor.calculateZscores(means,stdevs);
		TreeMap <String, Double> uaZscores = unknownAuthor.getZscores();
		int sz = knownAuthors.size();
		deltaValues = new ArrayList<Double>();
		TreeMap <String, ArrayList<Double>> frqs = mainSet.getFrequencies();
		for(int i=0; i<sz; i++){			
			TargetLexicon target = knownAuthors.get(i);
			target.calculateScores();
			target.calculateZscores(means,stdevs);
			TreeMap <String, Double> aZscores = target.getZscores();			
			List<String> stList = new  LinkedList<String>(aZscores.keySet());
			Collections.sort(stList);
			Iterator<String> it = stList.iterator(); 
			int lxSz = aZscores.size();
			double deltaValue=0.0;
			while(it.hasNext()){
				String key = it.next();
				Double azscObj = aZscores.get(key);
				if(azscObj==null){
					azscObj = new Double(0.0);
				}
				double azsc = azscObj.doubleValue();
				Double uazscObj = uaZscores.get(key);
				if(uazscObj==null){
					uazscObj = new Double(0.0);
				}
				double uazsc = uazscObj.doubleValue();
				double diff = azsc - uazsc;
				deltaValue += Math.abs(diff);
			}
			deltaValues.add(new Double(deltaValue/(double)lxSz));
		}
	}
	public void calculateCosineSimilarity(){
		TreeMap <String, Double> means = mainSet.getMeans();
		TreeMap <String, Double> stdevs = mainSet.getStdevs();
		unknownAuthor.calculateScores();
		unknownAuthor.calculateZscores(means,stdevs);
		TreeMap <String, Double> uaZscores = unknownAuthor.getZscores();
		int sz = knownAuthors.size();
		cosineSimilarities = new ArrayList<Double>();
		TreeMap <String, ArrayList<Double>> frqs = mainSet.getFrequencies();
		for(int i=0; i<sz; i++){			
			TargetLexicon target = knownAuthors.get(i);
			target.calculateScores();
			target.calculateZscores(means,stdevs);
			TreeMap <String, Double> aZscores = target.getZscores();			
			List<String> stList = new  LinkedList<String>(aZscores.keySet());
			Collections.sort(stList);
			Iterator<String> it = stList.iterator(); 
			int lxSz = aZscores.size();
			double cosineSimilarity=0.0;
			double aSquares=0.0;
			double uaSquares=0.0;
			while(it.hasNext()){
				String key = it.next();
				Double azscObj = aZscores.get(key);
				if(azscObj==null){
					azscObj = new Double(0.0);
				}
				double azsc = azscObj.doubleValue()/TrainingLexicon.DEFAULT_SCORE_SCALE;
				Double uazscObj = uaZscores.get(key);
				if(uazscObj==null){
					uazscObj = new Double(0.0);
				}
				double uazsc = uazscObj.doubleValue()/TrainingLexicon.DEFAULT_SCORE_SCALE;
				cosineSimilarity += azsc * uazsc;
				aSquares += azsc*azsc;
				uaSquares += uazsc*uazsc;
			}
			
			cosineSimilarities.add(new Double(cosineSimilarity/(aSquares*uaSquares)));
		}
	}
	public String deltaValueTable(){
		StringBuffer outStr = new StringBuffer();
		int sz = deltaValues.size();
		for(int i=0; i<sz; i++){
			outStr.append(authorNames.get(i));
			outStr.append("\t");
			Double delta = deltaValues.get(i);
			outStr.append(delta.toString());
			outStr.append("\n");
		}
 		return outStr.toString();
	}
	public String cosineSimilartyTable(){
		StringBuffer outStr = new StringBuffer();
		int sz = cosineSimilarities.size();
		for(int i=0; i<sz; i++){
			outStr.append(authorNames.get(i));
			outStr.append("\t");
			Double delta = cosineSimilarities.get(i);
			outStr.append(delta.toString());
			outStr.append("\n");
		}
 		return outStr.toString();
	}
	public String scoresTable(){
		StringBuffer outStr = new StringBuffer();
		int sz = knownAuthors.size();
		outStr.append("\tZ scores");
		for(int i=0; i<sz; i++){
			String name = authorNames.get(i);
			outStr.append("\t"+name);
		}
		outStr.append("\n");
		TreeMap <String, Integer> lexicon = unknownAuthor.getMfwList();
		if(lexicon==null){
			lexicon = unknownAuthor.lexicon;
		}
		List<String> stList = new  LinkedList<String>(lexicon.keySet());
		Collections.sort(stList);
		Iterator<String> it = stList.iterator(); 
		
		TreeMap <String, Double> uaZscrs = unknownAuthor.getZscores();
		while(it.hasNext()){
			String key = it.next();
			outStr.append(key+"\t");
			Double uazsObj = uaZscrs.get(key);
			if(uazsObj==null){
				uazsObj = new Double(0.0);
			}
			outStr.append(uazsObj.toString()); 
			
			for(int i=0; i<sz; i++){
				Double kazsObj = knownAuthors.get(i).getZscores().get(key);
				if(kazsObj==null){
					kazsObj = new Double(0.0);
				}
				outStr.append("\t"+kazsObj.toString()); 
			}
			outStr.append("\n");
		}
		return outStr.toString();	
	}
}
