package delta;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class DeltaProcedureApp extends JFrame implements ActionListener{

	/**
	 * 
	 */

	
	public static final String DEFAULT_TEXT_FILE_NAME = "text.xml";
	
	public static final String WHITE_SPACE = " :;?.,\n\r\t";
	public static final int DEFAULT_TRAINING_SECTION_SIZE = 1000;
	public static final int DEFAULT_NUM_MFW = 150;
	public static final String DEFAULT_CHUNK_SEPARATOR = "\n\n";
	public static final String DEFAULT_AUTHOR_NAME = "N.N.";
	private TreeMap <String,Integer> orderedLexicon;
	private TreeMap <String,Integer> orderedLexicon2;
	private TreeMap <String,int[]> trainingLexicon;
	private TreeMap <String,int[]> targetLexicon2;
	private TreeMap <String,Integer> targetLexicon;
	private TreeMap <String,Integer> authorLexicon;
	ArrayList<String> fqList;

	private TrainingLexicon tLexicon;
	private TargetLexicon uaLexicon;
	private TargetLexicon kaLexicon;
	private DeltaProcedure dProcedure; 
	private ArrayList<String> mfwList;
	private JButton collectTokensBtn; 
	private JButton saveTrainingData; 
	private JButton compareTypesBtn; 
	private JButton saveComparisionBtn; 
	private JButton buildTrigramBtn; 
	private JButton searchTrigramBtn; 
	private JButton collectTrainingData; 
	private JButton collectAuthorData; 
	private JButton collectTargetData; 
	private JButton saveScores; 
	private JButton processDelta; 
	private JButton resetData; 
	private JCheckBox useMFW; 
	private JCheckBox normalizeArabic; 
	private JCheckBox singleSectionScores; 
	private int tokenCount; 
	private int typeCount;
	private int sectionTokenCount; 
	private int targetTokenCount;
	private int authorTokenCount;
	private int trainingSectionSize;
	private int numMFW;


	private JTextField inputFld;
	private JTextField numMfwFld;
	private JTextField sizeOfTrainingSectionsFld;
	private JTextField trainingSectionSizeFld;
	private JTextArea textArea;	
	private JTextArea resultArea;	

	public DeltaProcedureApp(){
		this.setTitle("Delta Procedure");
		collectTokensBtn = new JButton("collect");
		collectTokensBtn.addActionListener(this); 
		collectTokensBtn.setActionCommand("c");
		
		buildTrigramBtn = new JButton("Build Trigrams");
		buildTrigramBtn.addActionListener(this); 
		buildTrigramBtn.setActionCommand("bt");
		
		searchTrigramBtn = new JButton("Search Trigrams");
		searchTrigramBtn.addActionListener(this); 
		searchTrigramBtn.setActionCommand("st");
		
		saveTrainingData = new JButton("save");
		saveTrainingData.addActionListener(this); 
		saveTrainingData.setActionCommand("std");

		compareTypesBtn = new JButton("collect second");
		compareTypesBtn.addActionListener(this); 
		compareTypesBtn.setActionCommand("2");

		saveComparisionBtn = new JButton("save comparision");
		saveComparisionBtn.addActionListener(this); 
		saveComparisionBtn.setActionCommand("b");
		
		collectTrainingData = new JButton("collect training data");
		collectTrainingData.addActionListener(this); 
		collectTrainingData.setActionCommand("td");
		
		collectAuthorData = new JButton("add known author");
		collectAuthorData.addActionListener(this); 
		collectAuthorData.setActionCommand("aud");

		collectTargetData = new JButton("collect unknown author");
		collectTargetData.addActionListener(this); 
		collectTargetData.setActionCommand("tgt");

		
		saveScores = new JButton("save scores");
		saveScores.addActionListener(this); 
		saveScores.setActionCommand("scrs");
		
		processDelta = new JButton("process Delta");
		processDelta.addActionListener(this); 
		processDelta.setActionCommand("pd");
				
		
		resetData = new JButton("reset all");
		resetData.addActionListener(this); 
		resetData.setActionCommand("rd");
		inputFld = new JTextField(); 
		inputFld.setText("N.N.");
		inputFld.setColumns(30);
		inputFld.setBorder(BorderFactory.createTitledBorder("Input"));
		numMfwFld = new JTextField(); 
		numMfwFld.setText("150");
		numMfwFld.setColumns(6);
		numMfwFld.setBorder(BorderFactory.createTitledBorder("Num MFW"));
		sizeOfTrainingSectionsFld = new JTextField(); 
		sizeOfTrainingSectionsFld.setText("1000");
		sizeOfTrainingSectionsFld.setColumns(6);
		sizeOfTrainingSectionsFld.setBorder(BorderFactory.createTitledBorder("Size Tr.Sect."));
		textArea = new JTextArea();
		textArea.setBorder(BorderFactory.createTitledBorder("Text"));
		textArea.setText("xxx yyy zzz zzz xxx yyy zzz");
		resultArea = new JTextArea();
		resultArea.setBorder(BorderFactory.createTitledBorder("Results"));
		resultArea.setText("Results");
		useMFW = new JCheckBox("Use MFW");
		useMFW.setSelected(true);
		normalizeArabic = new JCheckBox("Normalize Arabic");
		singleSectionScores = new JCheckBox("Single Section Scores");
		
		JPanel fldPn = new JPanel(); 
		JPanel prmPn = new JPanel(); 
		JPanel btnPn = new JPanel(); 
		fldPn.setLayout(new GridLayout(2,1));
		btnPn.setLayout(new GridLayout(1,3));
		JPanel allPn = new JPanel(); 
		allPn.setLayout(new BorderLayout());
		btnPn.add(collectTrainingData);
		btnPn.add(saveTrainingData);
		btnPn.add(collectAuthorData);
		btnPn.add(collectTargetData);
		btnPn.add(processDelta);
		btnPn.add(resetData);
		fldPn.add(textArea);
		fldPn.add(resultArea);
		allPn.add(btnPn, BorderLayout.SOUTH);
		allPn.add(fldPn, BorderLayout.CENTER);
		prmPn.add(inputFld);
		prmPn.add(numMfwFld);
		prmPn.add(sizeOfTrainingSectionsFld);
		prmPn.add(useMFW);
		prmPn.add(normalizeArabic);
		prmPn.add(singleSectionScores);
		allPn.add(prmPn, BorderLayout.NORTH);
		this.getContentPane().add(allPn);
	}
	
	public static void main(String[] args) {
		JFrame frm = new DeltaProcedureApp();
		frm.setBounds(50, 60, 1000, 600);
		frm.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if("td".equals(e.getActionCommand())) {
			loadTrainingText();
		}else if ("aud".equals(e.getActionCommand())) {
			loadAuthorText();
		}else if ("tgt".equals(e.getActionCommand())) {
			loadTargetText();
		}else if ("scrs".equals(e.getActionCommand())) {
			//processTargetScores();
		}else if ("std".equals(e.getActionCommand())) {
			saveTrainingData();
		}else if ("st".equals(e.getActionCommand())) {
			//searchTrigrams();
		}else if ("pd".equals(e.getActionCommand())) {
			processDelta();
		}else if ("rd".equals(e.getActionCommand())) {
			resetData();
		}		
	}

	public int processLineTargetData(String ln, TreeMap<String,Integer> lexicon){
		if(ln==null || lexicon == null){
			return 0;
		}
		int cnt = 0;
		StringTokenizer wdPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(wdPrsr.hasMoreTokens()){
			String wd = wdPrsr.nextToken();
			Integer sc = lexicon.get(wd);
			if(sc == null){
				lexicon.put(wd, new Integer(1));
			}else{
				sc++;
				lexicon.put(wd, sc);
			}
			cnt++;
		}
		return cnt;
	}

	public void loadTrainingText(){
		trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		int numOfSections = (int)Math.floor(cnt/trainingSectionSize);
		textArea.setText("Number of tokens = "+Integer.toString(cnt)+"; Number of sections = "+Integer.toString(numOfSections));
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineStr = null;
		int tknInSectionCnt = 0; 
		int sectionNum = 0; 
		tLexicon = new TrainingLexicon() ;
		StringBuffer strBuff = new StringBuffer();
		try {
			lineStr = bReader.readLine();
			if(normalizeArabic.isSelected()){
				lineStr = normalizeArabicLine(lineStr);
			}
			strBuff.append(lineStr);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			while(lineStr !=null){
				lineStr = bReader.readLine();
				if(normalizeArabic.isSelected()){
					lineStr = normalizeArabicLine(lineStr);
				}
				strBuff.append(lineStr);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		tLexicon.addText(strBuff.toString(), trainingSectionSize);
		tLexicon.collectStatistics();
		tLexicon.calculateFrequencies();
		if(useMFW.isSelected()){
			mfwList = tLexicon.getFequencyList(numMFW);
		}else{
			mfwList = tLexicon.getFequencyList();
		}
		dProcedure = new DeltaProcedure(tLexicon);
	} 
	public void saveTrainingData(){
		if(tLexicon==null){
			return;
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}

		TreeMap <String,ArrayList<Integer>> tStats = tLexicon.getStatistics();
		TreeMap <String,ArrayList<Double>> tFrqs = tLexicon.getFrequencies();
		ArrayList<Integer> tSizes = tLexicon.getChunkSizes();
		int textSize = tLexicon.getTextSize();
		Writer pWriter;
		String path_prefix = System.getProperty("user.dir");
		JFileChooser fc = new JFileChooser(new File(path_prefix));		 
		int rv = fc.showSaveDialog(DeltaProcedureApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
			        new FileOutputStream(selFile.getPath()), "UTF8"));
			pWriter.write("Total\t"+textSize);
			int chnkSz = tSizes.size();
			for(int i=0;i<chnkSz; i++){
				pWriter.write("\t"+tSizes.get(i).toString());
			}
			pWriter.write("\tMean\tStd\n");
			List<String> lxList = new  LinkedList<String>(tStats.keySet());
			Collections.sort(lxList);
			Iterator<String> it = lxList.iterator();
			while(it.hasNext()){
				String type = it.next();
				ArrayList<Integer> cArr= tStats.get(type);
				int cSz = cArr.size();
				pWriter.write(type);
				for(int i=0; i<cSz; i++){
					pWriter.write("\t"+cArr.get(i).toString());
				}
				ArrayList<Double> fArr = tFrqs.get(type);
				if(fArr!=null){
				int fSz = fArr.size();
					for(int i=0; i<fSz; i++){
						pWriter.write("\t"+fArr.get(i).toString());
					}
				}
				pWriter.write("\n");
			}
			pWriter.close();
			} catch (UnsupportedEncodingException e) {
			} catch (IOException e) {
			}
			fqList = null;
			if(useMFW.isSelected()){
				mfwList = tLexicon.getFequencyList(numMFW);
			}else{
				mfwList = tLexicon.getFequencyList();
			}
			int sz = mfwList.size();
			StringBuffer buff = new StringBuffer();
			for(int i=0; i<sz; i++){
				buff.append(mfwList.get(i)+"\n");
			}
			resultArea.setText(buff.toString());
			dProcedure = new DeltaProcedure(tLexicon);
	}

	
	public void  saveTrainingStatistics(TrainingLexicon  tLexicon){
		if(tLexicon==null){
			return;
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}

		tLexicon.collectStatistics();
		tLexicon.calculateFrequencies();
		TreeMap <String,ArrayList<Integer>> tStats = tLexicon.getStatistics();
		TreeMap <String,ArrayList<Double>> tFrqs = tLexicon.getFrequencies();
		ArrayList<Integer> tSizes = tLexicon.getChunkSizes();
		int textSize = tLexicon.getTextSize();
		Writer pWriter;
		String path_prefix = System.getProperty("user.dir");
		JFileChooser fc = new JFileChooser(new File(path_prefix));		 
		int rv = fc.showSaveDialog(DeltaProcedureApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
			        new FileOutputStream(selFile.getPath()), "UTF8"));
			pWriter.write("Total\t"+textSize);
			int chnkSz = tSizes.size();
			for(int i=0;i<chnkSz; i++){
				pWriter.write("\t"+tSizes.get(i).toString());
			}
			pWriter.write("\tMean\tStd\n");
			List<String> lxList = new  LinkedList<String>(tStats.keySet());
			Collections.sort(lxList);
			Iterator<String> it = lxList.iterator();
			while(it.hasNext()){
				String type = it.next();
				ArrayList<Integer> cArr= tStats.get(type);
				int cSz = cArr.size();
				pWriter.write(type);
				for(int i=0; i<cSz; i++){
					pWriter.write("\t"+cArr.get(i).toString());
				}
				ArrayList<Double> fArr = tFrqs.get(type);
				if(fArr!=null){
				int fSz = fArr.size();
					for(int i=0; i<fSz; i++){
						pWriter.write("\t"+fArr.get(i).toString());
					}
				}
				pWriter.write("\n");
			}
			pWriter.close();
			} catch (UnsupportedEncodingException e) {
			} catch (IOException e) {
			}
			fqList = null;
			if(useMFW.isSelected()){
				mfwList = tLexicon.getFequencyList(numMFW);
			}else{
				mfwList = tLexicon.getFequencyList();
			}
			int sz = mfwList.size();
			StringBuffer buff = new StringBuffer();
			for(int i=0; i<sz; i++){
				buff.append(mfwList.get(i)+"\n");
			}
			resultArea.setText(buff.toString());
			dProcedure = new DeltaProcedure(tLexicon);
	}
	public void loadAuthorText(){
		String authorName = inputFld.getText();
		if(authorName==null || authorName==""){
			authorName=DEFAULT_AUTHOR_NAME;
		}
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		int numOfSections = (int)Math.floor(cnt/trainingSectionSize);
		textArea.setText("Number of tokens = "+Integer.toString(cnt)+"; Number of sections = "+Integer.toString(numOfSections));
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineStr = null;
		kaLexicon = new TargetLexicon() ;
		StringBuffer strBuff = new StringBuffer();
		try {
			lineStr = bReader.readLine();
			if(normalizeArabic.isSelected()){
				lineStr = normalizeArabicLine(lineStr);
			}
			strBuff.append(lineStr);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			while(lineStr !=null){
				lineStr = bReader.readLine();
				if(normalizeArabic.isSelected()){
					lineStr = normalizeArabicLine(lineStr);
				}
				strBuff.append(lineStr);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}	
		kaLexicon.addText(strBuff.toString());
		if(mfwList!=null){
			kaLexicon.setMfwList(mfwList);
		}
		
		if(dProcedure!=null){
			dProcedure.addKnownAuthor(kaLexicon, authorName);
		}
	}
	public void loadTargetText(){
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		int numOfSections = (int)Math.floor(cnt/trainingSectionSize);
		textArea.setText("Number of tokens = "+Integer.toString(cnt)+"; Number of sections = "+Integer.toString(numOfSections));
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineStr = null;
		uaLexicon = new TargetLexicon() ;
		StringBuffer strBuff = new StringBuffer();
		try {
			lineStr = bReader.readLine();
			if(normalizeArabic.isSelected()){
				lineStr = normalizeArabicLine(lineStr);
			}
			strBuff.append(lineStr);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			while(lineStr !=null){
				lineStr = bReader.readLine();
				if(normalizeArabic.isSelected()){
					lineStr = normalizeArabicLine(lineStr);
				}
				strBuff.append(lineStr);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}	
		uaLexicon.addText(strBuff.toString());
		if(mfwList!=null){
			uaLexicon.setMfwList(mfwList);
		}
		if(dProcedure!=null){
			dProcedure.setUnknownAuthor(uaLexicon);
		}
	}
	public void processDelta(){
		if(dProcedure==null){
			return;
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}
		dProcedure.calculateDelta();
		dProcedure.calculateCosineSimilarity();
		StringBuffer out = new StringBuffer();
		out.append("================== Delta values ==================\n");
		out.append(dProcedure.deltaValueTable());
		out.append("================== Cosine similarity values ==================\n");
		out.append(dProcedure.cosineSimilartyTable());
		out.append("\n================== Delta values ==================\n");
		out.append(dProcedure.scoresTable());
		resultArea.setText(out.toString());
	}
	
	public void resetData(){
		
		fqList = null;

		tLexicon = null;
		uaLexicon = null;
		kaLexicon = null;
		dProcedure = null; 
		mfwList = null;
		
		
	}
	public int countLineTokens(String ln){
		if(ln==null){
			return 0;
		}
		StringTokenizer tknPrsr = new StringTokenizer(ln, WHITE_SPACE);
		return tknPrsr.countTokens();
	}
	public int countTokens(File selFile){
		JFileChooser fc = null;
		BufferedReader bReader = null;		

		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		int cnt = 0;
		String lineCnts = null;
		String ln = "";
		try {
			lineCnts = bReader.readLine();
			cnt= countLineTokens(lineCnts);
			
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				cnt += countLineTokens(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return cnt;
	}

	
	public String normalizeArabicLine(String ln){
		StringBuffer outStr = new StringBuffer();
		StringTokenizer wdPrsr = new StringTokenizer(ln, " .,:;.!?");
		boolean startFlag = true;
		while(wdPrsr.hasMoreTokens()){
			String wdStr = normalizeArabic(wdPrsr.nextToken());
			if(wdStr!=null && wdStr.length()>0){
				if(!startFlag){
					outStr.append(" ");
					
				}
				startFlag = false;
				outStr.append(wdStr);
			}
		}
		return outStr.toString();
	}
	public void normalizeArabicText(){
		String inStr = textArea.getText();
		if(inStr==null){
			return;
		}
		StringBuffer outStr = new StringBuffer();
		StringTokenizer lnPrsr = new StringTokenizer(inStr, "\n\r");
		while(lnPrsr.hasMoreTokens()){
			String lnStr = lnPrsr.nextToken();
			StringTokenizer wdPrsr = new StringTokenizer(lnStr, " .,:;.!?");
			boolean startFlag = true;
			while(wdPrsr.hasMoreTokens()){
				String wdStr = normalizeArabic(wdPrsr.nextToken());
				if(wdStr!=null && wdStr.length()>0){
					if(!startFlag){
						outStr.append(" ");
						
					}
					startFlag = false;
					outStr.append(wdStr);
				}
			}
			outStr.append("\n");
		}
		resultArea.setText(outStr.toString());
	}

	public static String normalizeArabic(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(!Character.isLetter(ch) || ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else if(ch=='\u0629'){
				outStr.append('\u0647');
			}else if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();

	}
	public static String extractHamzas(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}

	public void removeHamzas(){
		String inStr = textArea.getText();
		resultArea.setText(extractHamzas(inStr));
	}
	public void removeHamzasTamarbutaAlifmaksura(){
		String inStr = textArea.getText();
		resultArea.setText(removeAlifmaksura(removeTamarbuta(extractHamzas(inStr))));
	}

	public static String removeTamarbuta(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0629'){
				outStr.append('\u0647');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}
	public static String removeAlifmaksura(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}


}
