package delta;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

public class TargetData {
	public static String WHITE_SPACE_CHARACTERS = " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D";
	private int wordCount;
	private int mfwCount;
	private TreeMap<String,TargetDataEntry> lexicon;
	private TreeMap<String,TargetDataEntry> mfwLexicon;
	private TrainingData trainingData; 
	private double delta;
	
	public TargetData (TrainingData tData){
		trainingData = tData;
		lexicon = new TreeMap<String,TargetDataEntry>();
	}
	
	public void addText(String ln){
		if(ln==null){
			return;
		}
		StringTokenizer tokenPrsr = new StringTokenizer(ln, WHITE_SPACE_CHARACTERS);
		while(tokenPrsr.hasMoreTokens()){
			String tkn = tokenPrsr.nextToken();
			wordCount++;
			TargetDataEntry entry = lexicon.get(tkn);
			if(entry==null){	// new entry
				entry = new TargetDataEntry(tkn);
				lexicon.put(tkn, entry);
			}
			entry.increaseCount();
		}
	}
	public void calculateStatistics(){
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		//Collections.sort(lxKeyList);
		Iterator<String> lxIt = lxKeyList.iterator();
		double absZScoreSum = 0.0; 
		int entryCount = 0;
		while(lxIt.hasNext()){
			String key = lxIt.next();
			TargetDataEntry entry = lexicon.get(key);
			TrainingDataEntry tde= trainingData.getEntry(key);
			if(tde!=null){
				entry.calculateStatistics(this, tde);
				absZScoreSum += Math.abs(entry.getZscore());
				entryCount++;
			}
		}
		delta= absZScoreSum/(double)entryCount;
	}
	public void compare(TargetData authorData){
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> lxIt = lxKeyList.iterator();
		double absZScoreSum = 0.0; 
		int entryCount = 0;
		while(lxIt.hasNext()){
			String key = lxIt.next();
			TargetDataEntry entry = lexicon.get(key);
			TargetDataEntry tgde= authorData.getEntry(key);
			if(tgde!=null){
				//entry.calculateStatistics(this, tgde);
				absZScoreSum += Math.abs(entry.getZscore()-tgde.getZscore());
				entryCount++;
			}
		}
		delta= absZScoreSum/(double)entryCount;

		
	}
	public void compareMFW(TargetData authorData, String mfwStr){
		HashSet<String> mfwList = new HashSet<String>();
		StringTokenizer lnPrsr = new StringTokenizer(mfwStr, "\n\r");
		int tknCnt = 0; 
		while(lnPrsr.hasMoreTokens()){
			String tkn = lnPrsr.nextToken();
			if(tkn!=null && tkn.length()>0){
				mfwList.add(tkn);
			}
		}
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> lxIt = lxKeyList.iterator();
		double absZScoreSum = 0.0; 
		int entryCount = 0;
		mfwLexicon = new TreeMap<String,TargetDataEntry>();
		while(lxIt.hasNext()){
			String key = lxIt.next();
			if(mfwList.contains(key)){
				TargetDataEntry entry = lexicon.get(key);
				
				TargetDataEntry tgde= authorData.getEntry(key);
				if(tgde!=null){
					//entry.calculateStatistics(this, tgde);
					absZScoreSum += Math.abs(entry.getZscore()-tgde.getZscore());
					entryCount++;
				}
				mfwLexicon.put(key, entry);
				tknCnt++;
			}
		}
		delta= absZScoreSum/(double)entryCount;
		mfwCount = tknCnt;
	}
	public TargetDataEntry getEntry(String key){
		return lexicon.get(key);
	}


	public int getWordCount() {
		return wordCount;
	}

	public void setWordCount(int wordCount) {
		this.wordCount = wordCount;
	}

	public double getDelta() {
		return delta;
	}

	public void setDelta(double delta) {
		this.delta = delta;
	}
	public String toString(){
		StringBuffer outStr = new StringBuffer();
		outStr.append("Number of words = "+ wordCount+"\nDelta = "+delta+"\n");
		TreeMap<Double,Vector<TargetDataEntry>>frqList = new   TreeMap<Double,Vector<TargetDataEntry>>();
		List<String> keyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> it = keyList.iterator();
		while(it.hasNext()){
			String key = it.next();
			TargetDataEntry entry = lexicon.get(key);
			Double frq = new Double(-entry.getScore());
			Vector<TargetDataEntry> frqEntry = frqList.get(frq);
			if(frqEntry==null){
				Vector<TargetDataEntry> newEntry = new Vector<TargetDataEntry>();
				newEntry.add(entry);
				frqList.put(frq, newEntry);
			}else{
				frqEntry.add(entry);
			}
		}
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<TargetDataEntry> entries = frqList.get(key);
			for(int i=0; i< entries.size(); i++){
				outStr.append(entries.get(i).toString()+"\n");
			}
		}
		return outStr.toString();
	}
	public String mfwToString(){
		StringBuffer outStr = new StringBuffer();
		outStr.append("Number of words = "+ mfwCount+"\nDelta = "+delta+"\n");
		TreeMap<Double,Vector<TargetDataEntry>>frqList = new   TreeMap<Double,Vector<TargetDataEntry>>();
		List<String> keyList = new  LinkedList<String>(mfwLexicon.keySet());
		Iterator<String> it = keyList.iterator();
		while(it.hasNext()){
			String key = it.next();
			TargetDataEntry entry = mfwLexicon.get(key);
			Double frq = new Double(-entry.getScore());
			Vector<TargetDataEntry> frqEntry = frqList.get(frq);
			if(frqEntry==null){
				Vector<TargetDataEntry> newEntry = new Vector<TargetDataEntry>();
				newEntry.add(entry);
				frqList.put(frq, newEntry);
			}else{
				frqEntry.add(entry);
			}
		}
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<TargetDataEntry> entries = frqList.get(key);
			for(int i=0; i< entries.size(); i++){
				outStr.append(entries.get(i).toString()+"\n");
			}
		}
		return outStr.toString();
	}

}
