package delta;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;

public class TargetLexicon {

	TreeMap <String, Integer> lexicon;
	TreeMap <String, Integer> mfwList;
	TreeMap <String, Double> scores;
	TreeMap <String, Double> zscores;

	int textSize;
	double inverseTextSize;
	boolean useMfw; 
	
	public TargetLexicon(){
		lexicon = new TreeMap <String, Integer>();
		useMfw = false;
	}

	public void setMfwList(ArrayList<String> lst){
		mfwList = new TreeMap <String, Integer>();
		int sz = lst.size();
		for(int i=0; i<sz; i++){
			String tmp = lst.get(i);
			mfwList.put(tmp, new Integer(i));
		}
		useMfw = true;
	}
	public void addText(String txt){
		textSize = 0;
		StringTokenizer wdPrsr = new StringTokenizer(txt, TrainingLexicon.WHITE_SPACE_CHARACTERS); 
		while(wdPrsr.hasMoreTokens()){
			textSize++;
			String wd = wdPrsr.nextToken();
			if(useMfw){
				if(mfwList.containsKey(wd)){
					Integer lxm = lexicon.get(wd);
					if(lxm==null){
						lexicon.put(wd, new Integer(1));
					}else{
						lexicon.put(wd, new Integer(lxm.intValue()+1));
					}
				}
			}else{
				Integer lxm = lexicon.get(wd);
				if(lxm==null){
					lexicon.put(wd, new Integer(1));
				}else{
					lexicon.put(wd, new Integer(lxm.intValue()+1));
				}				
			}
			
		}
		inverseTextSize = 1.0/(double)textSize;
	}
	public void calculateScores(){
		scores = new TreeMap <String, Double>();
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		if(useMfw){
			lxKeyList = new  LinkedList<String>(mfwList.keySet());
		}
		
		Collections.sort(lxKeyList);
		Iterator<String> lxIt = lxKeyList.iterator();
		while(lxIt.hasNext()){
			String key = lxIt.next();
			Integer count = lexicon.get(key);
			if(count==null){
				count = new Integer(0);
			}
			Double score = new Double(count.doubleValue()*inverseTextSize*TrainingLexicon.DEFAULT_SCORE_SCALE);
			scores.put(key, score);
		}
	}
	
	public void calculateZscores(TreeMap <String, Double> mainSetMeans, TreeMap <String, Double> mainSetStdev){
		zscores = new TreeMap <String, Double>();
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		if(useMfw){
			lxKeyList = new  LinkedList<String>(mfwList.keySet());
		}
		Collections.sort(lxKeyList);
		Iterator<String> lxIt = lxKeyList.iterator();
		while(lxIt.hasNext()){
			String key = lxIt.next();
			Double score = scores.get(key);
			if(score==null){
				score = new Double(0.0);
			}
			Double mean = mainSetMeans.get(key);
			if(mean==null){
				mean = new Double(0.0);
			}
			Double stdev = mainSetStdev.get(key);
			if(stdev==null){
				stdev = new Double(1.0);
			}
			double zscore = (score.doubleValue()-mean.doubleValue())/stdev.doubleValue();
			zscores.put(key, new Double(zscore));
		}
	}
	public int getTextSize() {
		return textSize;
	}

	public void setTextSize(int textSize) {
		this.textSize = textSize;
	}

	public boolean isUseMfw() {
		return useMfw;
	}

	public void setUseMfw(boolean useMfw) {
		this.useMfw = useMfw;
	}

	public TreeMap<String, Integer> getLexicon() {
		return lexicon;
	}

	public void setLexicon(TreeMap<String, Integer> lexicon) {
		this.lexicon = lexicon;
	}

	public TreeMap<String, Integer> getMfwList() {
		return mfwList;
	}

	public void setMfwList(TreeMap<String, Integer> mfwList) {
		this.mfwList = mfwList;
	}

	public TreeMap<String, Double> getScores() {
		return scores;
	}

	public void setScores(TreeMap<String, Double> scores) {
		this.scores = scores;
	}

	public TreeMap<String, Double> getZscores() {
		return zscores;
	}

	public void setZscores(TreeMap<String, Double> zscores) {
		this.zscores = zscores;
	}
	
	
	

}
