package delta;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

public class TrainingData {
	public static String WHITE_SPACE_CHARACTERS = " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D";
	private int maxChunkSize; 
	private int wordCount;
	private int actualChunkNum;
	private int actualWordInChunk;
	private Vector<Integer> chunks;
	private TreeMap<String,TrainingDataEntry> lexicon;

	
	
	public TrainingData(int chnkSz){
		maxChunkSize = chnkSz;
		resetCounts();
		lexicon = new TreeMap<String,TrainingDataEntry>();
		chunks = new Vector<Integer>();
	}
	public void resetCounts(){
		wordCount =0; 
		actualChunkNum = 1; 
		actualWordInChunk = 0; 
	}
	public void addText(String ln){
		if(ln==null){
			return;
		}
		StringTokenizer tokenPrsr = new StringTokenizer(ln, WHITE_SPACE_CHARACTERS);
		while(tokenPrsr.hasMoreTokens()){
			actualWordInChunk++;
			wordCount++;
			if(actualWordInChunk>maxChunkSize){	// start new chunk
				actualChunkNum++;
				actualWordInChunk=1;
				chunks.add(new Integer(maxChunkSize));
			}
			String tkn = tokenPrsr.nextToken();
			TrainingDataEntry entry = lexicon.get(tkn);
			if(entry==null){	// new entry
				entry = new TrainingDataEntry(tkn);
				lexicon.put(tkn, entry);
			}
			entry.increaseCount(actualChunkNum);
		}
		chunks.add(new Integer(actualWordInChunk));
	}
	public void calculateStatistics(){
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		Collections.sort(lxKeyList);
		Iterator<String> lxIt = lxKeyList.iterator();
		int chunkNum = chunks.size();
		while(lxIt.hasNext()){
			String key = lxIt.next();
			TrainingDataEntry entry= lexicon.get(key);
			entry.finishCounting(chunkNum);
			entry.calculateMeanFrequency(wordCount);
			entry.calculateStdev(chunks);
		}

	}
	public TrainingDataEntry getEntry(String key){
		return lexicon.get(key);
	}
	public String toString(){
		StringBuffer outStr = new StringBuffer();
		TreeMap<Double,Vector<TrainingDataEntry>>frqList = new   TreeMap<Double,Vector<TrainingDataEntry>>();
		List<String> keyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> it = keyList.iterator();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String key = it.next();
			TrainingDataEntry entry = lexicon.get(key);
			Double frequency = new Double(-entry.getFrequency());
			Vector<TrainingDataEntry> frqEntry = frqList.get(frequency);
			if(frqEntry==null){
				Vector<TrainingDataEntry> newEntry = new Vector<TrainingDataEntry>();
				newEntry.add(entry);
				frqList.put(frequency, newEntry);
			}else{
				frqEntry.add(entry);
			}
		}
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<TrainingDataEntry> entries = frqList.get(key);
			for(int i=0; i< entries.size(); i++){
				outStr.append(entries.get(i).toString()+"\n");
			}
		}
		return outStr.toString();
	}
	public String toVerboseString(){
		StringBuffer outStr = new StringBuffer();
		TreeMap<Double,Vector<TrainingDataEntry>>frqList = new   TreeMap<Double,Vector<TrainingDataEntry>>();
		List<String> keyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> it = keyList.iterator();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String key = it.next();
			TrainingDataEntry entry = lexicon.get(key);
			Double frequency = new Double(-entry.getFrequency());
			Vector<TrainingDataEntry> frqEntry = frqList.get(frequency);
			if(frqEntry==null){
				Vector<TrainingDataEntry> newEntry = new Vector<TrainingDataEntry>();
				newEntry.add(entry);
				frqList.put(frequency, newEntry);
			}else{
				frqEntry.add(entry);
			}
		}
		List<Double> frqKeyList = new  LinkedList<Double>(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<TrainingDataEntry> entries = frqList.get(key);
			for(int i=0; i< entries.size(); i++){
				outStr.append(entries.get(i).toVerboseString()+"\n");
			}
		}
		return outStr.toString();
	}
	public String selectMFW(int numOfMFW){
		TreeMap<Double,Vector<TrainingDataEntry>>frqList = new   TreeMap<Double,Vector<TrainingDataEntry>>();
		List<String> keyList = new  LinkedList<String>(lexicon.keySet());
		Iterator<String> it = keyList.iterator();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String key = it.next();
			TrainingDataEntry entry = lexicon.get(key);
			Double frequency = new Double(-entry.getFrequency());
			Vector<TrainingDataEntry> frqEntry = frqList.get(frequency);
			if(frqEntry==null){
				Vector<TrainingDataEntry> newEntry = new Vector<TrainingDataEntry>();
				newEntry.add(entry);
				frqList.put(frequency, newEntry);
			}else{
				frqEntry.add(entry);
			}
		}
		StringBuffer outStr = new StringBuffer();
		List<Double> frqKeyList = new  LinkedList<Double>(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		int entryCnt = 0; 
		for(int i=0; entryCnt< numOfMFW && frqIt.hasNext(); i++){
			Double key = frqIt.next();
			Vector<TrainingDataEntry> entries = frqList.get(key);
			for(int j=0; j< entries.size(); j++){
				outStr.append(entries.get(j).getWord()+"\n");
				entryCnt++;
			}
		}
		return outStr.toString();
	}
}


