package delta;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class TrainingDataApp extends JFrame implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static final String DEFAULT_TEXT_FILE_NAME = "text.xml";
	
	public static final String WHITE_SPACE = " :;?.,\n\r\t";
	public static final int DEFAULT_TRAINING_SECTION_SIZE = 1000;
	public static final int DEFAULT_NUM_MFW = 150;
	public static final String DEFAULT_CHUNK_SEPARATOR = "\n\n";
	private TreeMap <String,Integer> orderedLexicon;
	private TreeMap <String,Integer> orderedLexicon2;
	private TreeMap <String,int[]> trainingLexicon;
	private TreeMap <String,int[]> targetLexicon2;
	private TreeMap <String,Integer> targetLexicon;
	private TreeMap <String,Integer> authorLexicon;
	ArrayList<String> fqList;
	// TreeMap <String,TokenFrequen;cy> comparLexicon;
	//private TrainingData trainingData;
	//private TargetData authorData;
	//private TargetData targetData;
	private TrainingLexicon tLexicon;
	private DeltaProcedure dProcedure; 
	private JButton collectTokensBtn; 
	private JButton saveTypesBtn; 
	private JButton compareTypesBtn; 
	private JButton saveComparisionBtn; 
	private JButton buildTrigramBtn; 
	private JButton searchTrigramBtn; 
	private JButton collectTrainingData; 
	private JButton collectAuthorData; 
	private JButton collectTargetData; 
	private JButton saveScores; 
	private JCheckBox useMFW; 
	private JCheckBox normalizeArabic; 
	private JCheckBox singleSectionScores; 
	private int tokenCount; 
	private int typeCount;
	private int sectionTokenCount; 
	private int targetTokenCount;
	private int authorTokenCount;
	private int trainingSectionSize;
	private int numMFW;


	private JTextField inputFld;
	private JTextField numMfwFld;
	private JTextField sizeOfTrainingSectionsFld;
	private JTextField trainingSectionSizeFld;
	private JTextArea textArea;	
	private JTextArea resultArea;	

	public TrainingDataApp(){
		this.setTitle("Delta");
		collectTokensBtn = new JButton("collect");
		collectTokensBtn.addActionListener(this); 
		collectTokensBtn.setActionCommand("c");
		
		buildTrigramBtn = new JButton("Build Trigrams");
		buildTrigramBtn.addActionListener(this); 
		buildTrigramBtn.setActionCommand("bt");
		
		searchTrigramBtn = new JButton("Search Trigrams");
		searchTrigramBtn.addActionListener(this); 
		searchTrigramBtn.setActionCommand("st");
		
		saveTypesBtn = new JButton("save");
		saveTypesBtn.addActionListener(this); 
		saveTypesBtn.setActionCommand("s");

		compareTypesBtn = new JButton("collect second");
		compareTypesBtn.addActionListener(this); 
		compareTypesBtn.setActionCommand("2");

		saveComparisionBtn = new JButton("save comparision");
		saveComparisionBtn.addActionListener(this); 
		saveComparisionBtn.setActionCommand("b");
		
		collectTrainingData = new JButton("collect training data");
		collectTrainingData.addActionListener(this); 
		collectTrainingData.setActionCommand("td");
		
		collectAuthorData = new JButton("add known author");
		collectAuthorData.addActionListener(this); 
		collectAuthorData.setActionCommand("aud");

		collectTargetData = new JButton("collect unknown author");
		collectTargetData.addActionListener(this); 
		collectTargetData.setActionCommand("tgd");

		
		saveScores = new JButton("save scores");
		saveScores.addActionListener(this); 
		saveScores.setActionCommand("scrs");

		
		
		inputFld = new JTextField(); 
		inputFld.setText("xxx_yyy");
		inputFld.setColumns(30);
		inputFld.setBorder(BorderFactory.createTitledBorder("Input"));
		numMfwFld = new JTextField(); 
		numMfwFld.setText("150");
		numMfwFld.setColumns(6);
		numMfwFld.setBorder(BorderFactory.createTitledBorder("Num MFW"));
		sizeOfTrainingSectionsFld = new JTextField(); 
		sizeOfTrainingSectionsFld.setText("1000");
		sizeOfTrainingSectionsFld.setColumns(6);
		sizeOfTrainingSectionsFld.setBorder(BorderFactory.createTitledBorder("Size Tr.Sect."));
		textArea = new JTextArea();
		textArea.setBorder(BorderFactory.createTitledBorder("Text"));
		textArea.setText("xxx yyy zzz zzz xxx yyy zzz");
		resultArea = new JTextArea();
		resultArea.setBorder(BorderFactory.createTitledBorder("Results"));
		resultArea.setText("Results");
		useMFW = new JCheckBox("Use MFW");
		normalizeArabic = new JCheckBox("Normalize Arabic");
		singleSectionScores = new JCheckBox("Single Section Scores");
		
		JPanel fldPn = new JPanel(); 
		JPanel prmPn = new JPanel(); 
		JPanel btnPn = new JPanel(); 
		fldPn.setLayout(new GridLayout(2,1));
		btnPn.setLayout(new GridLayout(1,11));
		JPanel allPn = new JPanel(); 
		allPn.setLayout(new BorderLayout());
		//btnPn.add(collectTokensBtn);
		btnPn.add(collectTrainingData);
		btnPn.add(collectAuthorData);
		btnPn.add(collectTargetData);
		btnPn.add(saveScores);
		btnPn.add(saveTypesBtn);
		btnPn.add(compareTypesBtn);
		btnPn.add(saveComparisionBtn);
		btnPn.add(buildTrigramBtn);
		btnPn.add(searchTrigramBtn);
		fldPn.add(textArea);
		fldPn.add(resultArea);
		allPn.add(btnPn, BorderLayout.SOUTH);
		allPn.add(fldPn, BorderLayout.CENTER);
		prmPn.add(inputFld);
		prmPn.add(numMfwFld);
		prmPn.add(sizeOfTrainingSectionsFld);
		prmPn.add(useMFW);
		prmPn.add(normalizeArabic);
		prmPn.add(singleSectionScores);
		allPn.add(prmPn, BorderLayout.NORTH);
		this.getContentPane().add(allPn);
	}
	
	public static void main(String[] args) {
		JFrame frm = new TrainingDataApp	();
		frm.setBounds(50, 60, 1000, 600);
		frm.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if ("c".equals(e.getActionCommand())) {
	        collectTypes();
		}else if ("td".equals(e.getActionCommand())) {
			collectTrainingData4();
		}else if ("aud".equals(e.getActionCommand())) {
			collectAuthorData();
		}else if ("tgd".equals(e.getActionCommand())) {
			 collectTargetData();
		}else if ("scrs".equals(e.getActionCommand())) {
			processTargetScores();
		}else if ("s".equals(e.getActionCommand())) {
			saveTokenCount();
		}else if ("2".equals(e.getActionCommand())) {
			openSecond();
		}else if ("b".equals(e.getActionCommand())) {
			saveSecondCount();
		}else if ("bt".equals(e.getActionCommand())) {
			//buildTrigrams();	
		}else if ("st".equals(e.getActionCommand())) {
			//searchTrigrams();
		}		
	}

	public void collectTypes(){
		TreeMap<String,Integer> lexicon = new   TreeMap<String,Integer>();
		
		loadText(lexicon);
		orderedLexicon = lexicon;
	}
	public void collectTrainingData3(){
		loadTrainingText3();
	}
	public void collectTrainingData4(){
		loadTrainingText4();
	}
	public void saveTokenCount(){
		//saveTypes(orderedLexicon);
	}
	public void collectTargetData(){
		loadTargetText(targetLexicon);
	}
	public void collectAuthorData(){
		loadAuthorText(authorLexicon);
		processAuthorScores();
	}
	public void openSecond(){
		TreeMap<String,Integer> lexicon = new   TreeMap<String,Integer>();	
		loadText(lexicon);
		orderedLexicon2 = lexicon;	
	}
	public void saveSecondCount(){
		//saveTypes(orderedLexicon2);
		//saveComparision(orderedLexicon, orderedLexicon2);
	}


	public void loadText(TreeMap<String,Integer> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		if(lexicon==null){
			lexicon = new   TreeMap<String,Integer>();
		}
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		//String path_std = path_prefix+"/text.txt";		
		 JFileChooser fc = new JFileChooser(new File(path_prefix));
		 fc.showOpenDialog(this);
		 File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			processLine(lineCnts, lexicon);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				processLine(lineCnts, lexicon);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		inputFld.setText("Ratio: "+tokenCount+"/"+typeCount+" = "+((double) tokenCount/typeCount));
	}


	public void processLine(String ln, TreeMap<String,Integer> lexicon){
		
		if(ln==null){
			return;
		}
		StringTokenizer tokenPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(tokenPrsr.hasMoreTokens()){
			String tkn = tokenPrsr.nextToken();
			Integer cnt = lexicon.get(tkn);
			if(cnt!=null){
				lexicon.remove(tkn);
				lexicon.put(tkn, new Integer(cnt+1));
				tokenCount++;
			}else{
				lexicon.put(tkn, new Integer(1));
				tokenCount++;
				typeCount++;
			}
		}
	}
	public void loadTrainingText(TreeMap<String,Vector<Integer>> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		sectionTokenCount = 0; 
		Vector<Integer> sectionTokenSizes = new Vector<Integer>();
		int trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		int sectionNum = 1;
		//if(lexicon==null){
			lexicon = new   TreeMap<String,Vector<Integer>>();
		//}
		BufferedReader bReader = null;		
		String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_home_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			sectionTokenCount = processLineTrainingData(lineCnts, lexicon, sectionTokenCount, trainingSectionSize,sectionTokenSizes);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				sectionTokenCount = processLineTrainingData(lineCnts, lexicon, sectionTokenCount, trainingSectionSize, sectionTokenSizes);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		sectionTokenSizes.add(new Integer(sectionTokenCount));
		
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		
		//List<String> frqList = new  LinkedList();
		TreeMap<Double,Vector<String>>frqList = new   TreeMap<Double,Vector<String>>();
		while(it.hasNext()){
			//StringBuffer outStr = new StringBuffer();
			String type = it.next();
			Vector<Integer> cnt = lexicon.get(type);
			double mean = getMeanFrequency(cnt, sectionTokenSizes);
			double stdev = getStdev(cnt, mean, sectionTokenSizes);
			String entryStr = type+"\t"+mean+"\t"+stdev ;
			Double frq = new Double(-mean);
			Vector<String> entry = frqList.get(frq);
			if(entry==null){
				Vector<String> newEntry = new Vector<String>();
				newEntry.add(entryStr);
				frqList.put(frq, newEntry);
			}else{
				entry.add(entryStr);
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Double(mean),type+"\t"+mean+"\t"+stdev );
		}
		List<Double> frqKeyList = new  LinkedList(frqList.keySet());
		Collections.sort(frqKeyList);
		Iterator<Double> frqIt = frqKeyList.iterator();
		StringBuffer outStr = new StringBuffer();
		while(frqIt.hasNext()){
			Double key = frqIt.next();
			Vector<String> vls = frqList.get(key);
			for(int i=0; i< vls.size(); i++){
				outStr.append(vls.get(i)+"\n");
			}
			//outStr.append(type+"\t"+mean+"\t"+stdev);
			//frqList.put(new Integer(mean),type+"\t"+mean+"\t"+stdev )
		}
	
		

		resultArea.setText(outStr.toString());
	}

	public int processLineTrainingData(String ln, TreeMap<String,Vector<Integer>> lexicon, int sectionTokenCount, int trainingSectionSize, Vector<Integer> sectionTokenSizes){
		if(ln==null){
			return sectionTokenCount;
		}
		int sectionNum = sectionTokenSizes.size()+1;
		StringTokenizer tokenPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(tokenPrsr.hasMoreTokens()){
			sectionTokenCount++;
			if(sectionTokenCount>trainingSectionSize){	// start new section
				sectionNum++;
				sectionTokenCount=1;
				sectionTokenSizes.add(new Integer(trainingSectionSize));
				sectionNum = sectionTokenSizes.size();
			}
			String tkn = tokenPrsr.nextToken();
			Vector<Integer> cnt = lexicon.get(tkn);
			if(cnt==null){	// new type
				cnt = new Vector<Integer>();
			}
			if(cnt.size()<sectionNum){	// no counter for this section
				for(int i=cnt.size(); i<sectionNum-1; i++){	// missing counters in between
					cnt.add(new Integer(0));
				}
				cnt.add(new Integer(1)); // counter for this section
			}else{						// counter for this section exists
				Integer newCnt = new Integer(cnt.get(sectionNum-1)+1);
				cnt.set(sectionNum-1, newCnt);
				//cntr++;
			}
			lexicon.put(tkn, cnt);
			tokenCount++;
		}
		return sectionTokenCount; 
	}

	public double getMeanFrequency(Vector<Integer> cnt, Vector<Integer>  sectionTokenSizes){
		double count = 0.0;
		double allToken = 0.0;
		int sz = cnt.size();
		int sctSz = sectionTokenSizes.size();
		for(int i=0; i<sz; i++){
			count += cnt.get(i);
		}
		for(int i=0; i<sctSz; i++){
			allToken += (double)sectionTokenSizes.get(i);
		}
		double mean = count/allToken;
		return mean;
	}
	public double getStdev(Vector<Integer> cnt, double mean,Vector<Integer>  sectionTokenSizes){
		double stdev = 0.0;
		int sz = cnt.size();
		for(int i=0; i<sz; i++){
			double invNum = 1.0/(double)sectionTokenSizes.get(i);
			double tmp = mean- invNum*cnt.get(i);
			stdev += tmp*tmp;
		}
		return Math.sqrt(stdev/(sz-1));
	}
	
	
	public void loadTargetText(TreeMap<String,Integer> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		if(lexicon==null){
			lexicon = new   TreeMap<String,Integer>();
		}
		BufferedReader bReader = null;
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			tokenCount += processLineTargetData(lineCnts, lexicon);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				tokenCount += processLineTargetData(lineCnts, lexicon);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//sectionTokenSizes.add(new Integer(sectionTokenCount));
		
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		StringBuffer outStr = new StringBuffer();
		//List<String> frqList = new  LinkedList();
		TreeMap<Double,Vector<String>>frqList = new   TreeMap<Double,Vector<String>>();
		while(it.hasNext()){
			String type = it.next();
			Integer cnt = lexicon.get(type);
			outStr.append(type+"\t"+cnt.toString()+"\n");
		}
		targetTokenCount = tokenCount;
		typeCount = lexicon.size();
		textArea.setText("Token = "+Integer.toString(tokenCount)+"; Type = "+ Integer.toString(typeCount));
		resultArea.setText(outStr.toString());
		targetLexicon = lexicon;
	}
	
	public void processTargetScores(){
		if(targetLexicon == null || trainingLexicon == null){
			return;
		}
		int sz = targetLexicon.size();
		TreeMap<String,double[]>  scoreTable = new TreeMap<String,double[]>();
		List<String> lxList = new  LinkedList(trainingLexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		while(it.hasNext()){
			String type = it.next();
			int [] trCnts = trainingLexicon.get(type);
			int sctSz = trCnts.length;
			double trMean = 100.0*getMean(trCnts)/(double)trainingSectionSize;
			double trStdev = 100.0*getStdev(trCnts)/(double)trainingSectionSize;
			Integer tgCnt = targetLexicon.get(type);
			double tgScore = 0.0;
			if(tgCnt != null){
				tgScore = 100.0*tgCnt.intValue()/(double)targetTokenCount;
			}
			double zScore = (tgScore-trMean)/trStdev;
			double [] data = new double[4];
			data[0] = trMean;
			data[1] = trStdev;
			data[2] = tgScore;
			data[3] = zScore;
			scoreTable.put(type, data);
		}
		saveScores(scoreTable);
	}
	public void  saveScores(TreeMap<String,double[]>  lexicon){
		if(lexicon==null){
			return;
		}
		Writer pWriter;
		//String path_home_prefix = System.getProperty("user.home");		
		String path_prefix = System.getProperty("user.dir");
		JFileChooser fc = new JFileChooser(new File(path_prefix));		 
		int rv = fc.showSaveDialog(TrainingDataApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
			        new FileOutputStream(selFile.getPath()), "UTF8"));
			List<String> lxList = new  LinkedList(lexicon.keySet());
			Collections.sort(lxList);
			Iterator<String> it = lxList.iterator();
			while(it.hasNext()){
				String type = it.next();
				double[] cnts = lexicon.get(type);
				pWriter.write(type);
				pWriter.write("\t"+Double.toString(cnts[0]));
				pWriter.write("\t"+Double.toString(cnts[1]));
				pWriter.write("\t"+Double.toString(cnts[2]));
				pWriter.write("\t"+Double.toString(cnts[3]));
				pWriter.write("\n");
			}
			pWriter.close();
			} catch (UnsupportedEncodingException e) {
			} catch (IOException e) {
			}
	}

	public int processLineTargetData(String ln, TreeMap<String,Integer> lexicon){
		if(ln==null || lexicon == null){
			return 0;
		}
		int cnt = 0;
		StringTokenizer wdPrsr = new StringTokenizer(ln, " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D");
		while(wdPrsr.hasMoreTokens()){
			String wd = wdPrsr.nextToken();
			Integer sc = lexicon.get(wd);
			if(sc == null){
				lexicon.put(wd, new Integer(1));
			}else{
				sc++;
				lexicon.put(wd, sc);
			}
			cnt++;
		}
		return cnt;
	}
	public void loadTrainingText3(){
		//Vector<Integer> sectionTokenSizes = new Vector<Integer>();
	trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		//tData = new TrainingData(trainingSectionSize);
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		int numOfSections = (int)Math.floor(cnt/trainingSectionSize);
		textArea.setText("Number of tokens = "+Integer.toString(cnt)+"; Number of sections = "+Integer.toString(numOfSections));
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineStr = null;
		int tknInSectionCnt = 0; 
		int sectionNum = 0; 
		trainingLexicon = new TreeMap<String,int[]>() ;

		try {
			lineStr = bReader.readLine();
			if(normalizeArabic.isSelected()){
				lineStr = normalizeArabicLine(lineStr);
			}
			int tokenRead = readLineToken(lineStr, sectionNum, tknInSectionCnt, trainingSectionSize, numOfSections, trainingLexicon);
			sectionNum += (int)Math.floor(tokenRead/trainingSectionSize);
			tknInSectionCnt = tokenRead%trainingSectionSize;
			
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			while(lineStr !=null && sectionNum<numOfSections){
				lineStr = bReader.readLine();
				if(normalizeArabic.isSelected()){
					lineStr = normalizeArabicLine(lineStr);
				}
				int tokenRead = readLineToken(lineStr, sectionNum, tknInSectionCnt, trainingSectionSize, numOfSections, trainingLexicon);
				sectionNum += (int)Math.floor(tokenRead/trainingSectionSize);
				tknInSectionCnt = tokenRead%trainingSectionSize;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		//String outStr = trainingCountsToString(trainingLexicon);
		saveTrainingCounts(trainingLexicon);
		//resultArea.setText(outStr);

		
	}
	public void loadTrainingText4(){
		trainingSectionSize = DEFAULT_TRAINING_SECTION_SIZE;
		try{
			String tmpStr = sizeOfTrainingSectionsFld.getText();
			trainingSectionSize = Integer.parseInt(tmpStr);
		}catch(NumberFormatException e){
			
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}
		BufferedReader bReader = null;		
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		
		int cnt = countTokens(selFile);
		int numOfSections = (int)Math.floor(cnt/trainingSectionSize);
		textArea.setText("Number of tokens = "+Integer.toString(cnt)+"; Number of sections = "+Integer.toString(numOfSections));
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineStr = null;
		int tknInSectionCnt = 0; 
		int sectionNum = 0; 
		tLexicon = new TrainingLexicon() ;
		StringBuffer strBuff = new StringBuffer();
		try {
			lineStr = bReader.readLine();
			if(normalizeArabic.isSelected()){
				lineStr = normalizeArabicLine(lineStr);
			}
			strBuff.append(lineStr);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			while(lineStr !=null){
				lineStr = bReader.readLine();
				if(normalizeArabic.isSelected()){
					lineStr = normalizeArabicLine(lineStr);
				}
				strBuff.append(lineStr);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		tLexicon.addText(strBuff.toString(), trainingSectionSize);
		saveTrainingStatistics(tLexicon);
	}
	public void  saveTrainingStatistics(TrainingLexicon  tLexicon){
		if(tLexicon==null){
			return;
		}
		numMFW = DEFAULT_NUM_MFW; 
		String numMfwStr = numMfwFld.getText();
		try{
			numMFW = Integer.parseInt(numMfwStr);
		}catch(NumberFormatException e){
			
		}

		tLexicon.collectStatistics();
		tLexicon.calculateFrequencies();
		TreeMap <String,ArrayList<Integer>> tStats = tLexicon.getStatistics();
		TreeMap <String,ArrayList<Double>> tFrqs = tLexicon.getFrequencies();
		ArrayList<Integer> tSizes = tLexicon.getChunkSizes();
		int textSize = tLexicon.getTextSize();
		Writer pWriter;
		String path_prefix = System.getProperty("user.dir");
		JFileChooser fc = new JFileChooser(new File(path_prefix));		 
		int rv = fc.showSaveDialog(TrainingDataApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
			        new FileOutputStream(selFile.getPath()), "UTF8"));
			pWriter.write("Total\t"+textSize);
			int chnkSz = tSizes.size();
			for(int i=0;i<chnkSz; i++){
				pWriter.write("\t"+tSizes.get(i).toString());
			}
			pWriter.write("\tMean\tStd\n");
			List<String> lxList = new  LinkedList<String>(tStats.keySet());
			Collections.sort(lxList);
			Iterator<String> it = lxList.iterator();
			while(it.hasNext()){
				String type = it.next();
				ArrayList<Integer> cArr= tStats.get(type);
				int cSz = cArr.size();
				pWriter.write(type);
				for(int i=0; i<cSz; i++){
					pWriter.write("\t"+cArr.get(i).toString());
				}
				ArrayList<Double> fArr = tFrqs.get(type);
				if(fArr!=null){
				int fSz = fArr.size();
					for(int i=0; i<fSz; i++){
						pWriter.write("\t"+fArr.get(i).toString());
					}
				}
				pWriter.write("\n");
			}
			pWriter.close();
			} catch (UnsupportedEncodingException e) {
			} catch (IOException e) {
			}
			fqList = null;
			if(useMFW.isSelected()){
				fqList = tLexicon.getFequencyList(numMFW);
			}else{
				fqList = tLexicon.getFequencyList();
			}
			int sz = fqList.size();
			StringBuffer buff = new StringBuffer();
			for(int i=0; i<sz; i++){
				buff.append(fqList.get(i)+"\n");
			}
			resultArea.setText(buff.toString());
			dProcedure = new DeltaProcedure(tLexicon);
	}

	public void  saveTrainingCounts(TreeMap<String,int[]>  lexicon){
		if(lexicon==null){
			return;
		}
		Writer pWriter;
		//String path_home_prefix = System.getProperty("user.home");
		String path_prefix = System.getProperty("user.dir");
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		int rv = fc.showSaveDialog(TrainingDataApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
			        new FileOutputStream(selFile.getPath()), "UTF8"));
			List<String> lxList = new  LinkedList(lexicon.keySet());
			Collections.sort(lxList);
			Iterator<String> it = lxList.iterator();
			while(it.hasNext()){
				String type = it.next();
				int[] cnts = lexicon.get(type);
				int sz = cnts.length;
				pWriter.write(type);
				//double mn = getMean(cnts)/(double)sz;
				double mn = getMean(cnts);
				pWriter.write("\t"+Double.toString(mn));
				double stdv = getStdev(cnts);
				pWriter.write("\t"+Double.toString(stdv));
				if(singleSectionScores.isSelected()){
					for(int i=0; i<sz; i++){
						pWriter.write("\t"+cnts[i]);
					}
				}
				pWriter.write("\n");
			}
			pWriter.close();
			} catch (UnsupportedEncodingException e) {
			} catch (IOException e) {
			}
	}
	public static double getMean(int [] cnts){
		if(cnts==null){
			return 0;
		}
		int sz = cnts.length;
		double mn = 0.0; 
		for(int i=0; i<sz; i++){
			mn += cnts[i];
		}
		mn /= sz;
		return mn;
	}
	public static double getStdev(int [] cnts){
		if(cnts==null){
			return 0;
		}
		int sz = cnts.length;
		double mean = getMean(cnts);
		double stdev = 0.0;
		double invNum = 1.0/(double)sz;
		for(int i=0; i<sz; i++){
			double tmp = mean- invNum*cnts[i];
			stdev += tmp*tmp;
		}
		return Math.sqrt(stdev/(sz-1));	
	}
	/*
	public static double getStdev(int [] cnts, int smplSz){
		if(cnts==null){
			return 0;
		}
		
		double invSmplSz = 1.0/(double)smplSz;
		int sz = cnts.length;
		double mean = getMean(cnts);
		double stdev = 0.0;
		double invNum = 1.0/(double)sz;
		for(int i=0; i<sz; i++){
			double tmp = mean- invNum*cnts[i];
			stdev += tmp*tmp;
		}
		return Math.sqrt(stdev/(sz-1));	
	}
	*/

	
public void saveTypes(TreeMap<String,Integer> lexicon){
		if(lexicon==null){
			return;
		}
		Writer pWriter;
		String path_home_prefix = System.getProperty("user.home");		
		 JFileChooser fc = new JFileChooser(new File(path_home_prefix));		 
		int rv = fc.showSaveDialog(TrainingDataApp.this);
		if(rv!= JFileChooser.APPROVE_OPTION){
				return;
		}
		File selFile = fc.getSelectedFile();
		 fc.remove(this);
		 fc = null;
		 if(selFile==null){
			 return;
		 }
		try {
			pWriter = new BufferedWriter(new OutputStreamWriter(
		        new FileOutputStream(selFile.getPath()), "UTF8"));
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		while(it.hasNext()){
			String type = it.next();
			Integer cnt = lexicon.get(type);
			pWriter.write(type+"\t"+cnt.toString()+"\n");
		}
		pWriter.close();
		} catch (UnsupportedEncodingException e) {
		} catch (IOException e) {
		}
	}
	public String trainingCountsToString(TreeMap<String,int[]>  trainingLexicon){
		StringBuffer out = new StringBuffer();
		int sz = trainingLexicon.size();
		List<String> frqKeyList = new  LinkedList(trainingLexicon.keySet());
		Collections.sort(frqKeyList);
		Iterator<String> frqIt = frqKeyList.iterator();
		while(frqIt.hasNext()){
			String key = frqIt.next();
			out.append(key);
			int [] cnts = trainingLexicon.get(key);
			int secSz = cnts.length;
			for(int i=0; i< secSz; i++){
				out.append("\t"+Integer.toString(cnts[i]));
			}
			out.append("\n");
		}
		return out.toString();
	}
	public int readLineToken(String ln, int sectionNum, int tknInSection, int trainingSectionSize, int numOfSections, TreeMap<String,int[]> trainingLexicon){
		int cnt = 0; 
		if(ln==null){
			return 0;
		}
		StringTokenizer tknPrsr = new StringTokenizer(ln, WHITE_SPACE);
		while(tknPrsr.hasMoreTokens()){
			if(tknInSection==trainingSectionSize){
				sectionNum++;
				if(sectionNum==numOfSections){
					return cnt;
				}
				tknInSection=0;
			}
			String tkn = tknPrsr.nextToken();
			int [] cnts = trainingLexicon.get(tkn);
			if(cnts==null){
				cnts = new int [numOfSections];
				cnts[sectionNum] = 1;
				trainingLexicon.put(tkn, cnts);
			}else{
				cnts[sectionNum]++;
			}
			cnt++;
			tknInSection++;
		}
		
		
		return cnt;
	}
	public int countLineTokens(String ln){
		if(ln==null){
			return 0;
		}
		StringTokenizer tknPrsr = new StringTokenizer(ln, WHITE_SPACE);
		return tknPrsr.countTokens();
	}
	public int countTokens(File selFile){
		JFileChooser fc = null;
		BufferedReader bReader = null;		

		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		int cnt = 0;
		String lineCnts = null;
		String ln = "";
		try {
			lineCnts = bReader.readLine();
			cnt= countLineTokens(lineCnts);
			
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				cnt += countLineTokens(lineCnts);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return cnt;
	}
	
	public void loadAuthorText(TreeMap<String,Integer> lexicon){
		tokenCount = 0; 
		typeCount = 0; 
		if(lexicon==null){
			lexicon = new   TreeMap<String,Integer>();
		}
		BufferedReader bReader = null;
		String path_prefix = System.getProperty("user.dir");
		//String path_home_prefix = System.getProperty("user.home");		
		JFileChooser fc = new JFileChooser(new File(path_prefix));
		fc.showOpenDialog(this);
		File selFile = fc.getSelectedFile();
		fc.remove(this);
		fc = null;
		try {
			try {
				bReader = new BufferedReader(new InputStreamReader(new FileInputStream(selFile.getPath()), "UTF8"));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String lineCnts = null;
		try {
			lineCnts = bReader.readLine();
			tokenCount += processLineTargetData(lineCnts, lexicon);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		try {
			while(lineCnts !=null){
				lineCnts = bReader.readLine();
				tokenCount += processLineTargetData(lineCnts, lexicon);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			bReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//sectionTokenSizes.add(new Integer(sectionTokenCount));
		
		List<String> lxList = new  LinkedList(lexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		StringBuffer outStr = new StringBuffer();
		//List<String> frqList = new  LinkedList();
		TreeMap<Double,Vector<String>>frqList = new   TreeMap<Double,Vector<String>>();
		while(it.hasNext()){
			String type = it.next();
			Integer cnt = lexicon.get(type);
			outStr.append(type+"\t"+cnt.toString()+"\n");
		}
		authorTokenCount = tokenCount;
		typeCount = lexicon.size();
		textArea.setText("Token = "+Integer.toString(tokenCount)+"; Type = "+ Integer.toString(typeCount));
		resultArea.setText(outStr.toString());
		authorLexicon = lexicon;
	}
	public void processAuthorScores(){
		if(authorLexicon == null || targetLexicon == null || trainingLexicon == null){
			return;
		}
		int sz = authorLexicon.size();
		TreeMap<String,double[]>  scoreTable = new TreeMap<String,double[]>();
		List<String> lxList = new  LinkedList(trainingLexicon.keySet());
		Collections.sort(lxList);
		Iterator<String> it = lxList.iterator();
		while(it.hasNext()){
			String type = it.next();
			int [] trCnts = trainingLexicon.get(type);
			int sctSz = trCnts.length;
			double trMean = 100.0*getMean(trCnts)/(double)trainingSectionSize;
			double trStdev = 100.0*getStdev(trCnts)/(double)trainingSectionSize;
			Integer tgCnt = targetLexicon.get(type);
			double tgScore = 0.0;
			if(tgCnt != null){
				tgScore = 100.0*tgCnt.intValue()/(double)authorTokenCount;
			}
			double zScore = (tgScore-trMean)/trStdev;
			double [] data = new double[4];
			data[0] = trMean;
			data[1] = trStdev;
			data[2] = tgScore;
			data[3] = zScore;
			scoreTable.put(type, data);
		}
		saveScores(scoreTable);
	}

	
	/*
	public static String normalizeArabic(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(!Character.isLetter(ch) || ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else if(ch=='\u0629'){
				outStr.append('\u0647');
			}else if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();

	}
	*/
	public String normalizeArabicLine(String ln){
		StringBuffer outStr = new StringBuffer();
		StringTokenizer wdPrsr = new StringTokenizer(ln, " .,:;.!?");
		boolean startFlag = true;
		while(wdPrsr.hasMoreTokens()){
			String wdStr = normalizeArabic(wdPrsr.nextToken());
			if(wdStr!=null && wdStr.length()>0){
				if(!startFlag){
					outStr.append(" ");
					
				}
				startFlag = false;
				outStr.append(wdStr);
			}
		}
		return outStr.toString();
	}
	public void normalizeArabicText(){
		String inStr = textArea.getText();
		if(inStr==null){
			return;
		}
		StringBuffer outStr = new StringBuffer();
		StringTokenizer lnPrsr = new StringTokenizer(inStr, "\n\r");
		while(lnPrsr.hasMoreTokens()){
			String lnStr = lnPrsr.nextToken();
			StringTokenizer wdPrsr = new StringTokenizer(lnStr, " .,:;.!?");
			boolean startFlag = true;
			while(wdPrsr.hasMoreTokens()){
				String wdStr = normalizeArabic(wdPrsr.nextToken());
				if(wdStr!=null && wdStr.length()>0){
					if(!startFlag){
						outStr.append(" ");
						
					}
					startFlag = false;
					outStr.append(wdStr);
				}
			}
			outStr.append("\n");
		}
		resultArea.setText(outStr.toString());
	}

	public static String normalizeArabic(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(!Character.isLetter(ch) || ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else if(ch=='\u0629'){
				outStr.append('\u0647');
			}else if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();

	}
	public static String extractHamzas(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0622' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u0627');
			}else if(ch=='\u0624'){
				outStr.append('\u0648');
			}else if(ch=='\u0626' || ch=='\u0623' || ch=='\u0625'){
				outStr.append('\u064A');
			}else if(ch=='\u0621' || ch=='\u0653' || ch=='\u0654'|| ch=='\u0655'){
				;
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}

	public void removeHamzas(){
		String inStr = textArea.getText();
		resultArea.setText(extractHamzas(inStr));
	}
	public void removeHamzasTamarbutaAlifmaksura(){
		String inStr = textArea.getText();
		resultArea.setText(removeAlifmaksura(removeTamarbuta(extractHamzas(inStr))));
	}

	public static String removeTamarbuta(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0629'){
				outStr.append('\u0647');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}
	public static String removeAlifmaksura(String str){
		if(str==null){
			return null;
		}
		StringBuffer outStr = new StringBuffer();
		int ln = str.length();
		for(int i=0; i<ln; i++){
			char ch = str.charAt(i);
			if(ch=='\u0649'){
				outStr.append('\u064A');
			}else{
				outStr.append(ch);
			}
		}
		return outStr.toString();
	}


}
