package delta;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;

public class TrainingLexicon {
	public static String WHITE_SPACE_CHARACTERS = " ,.;:!?/|\t\r\u061B\u061F\u060C\u060D";
	public static final double DEFAULT_SCORE_SCALE = 100.0;
	
	TreeMap <String, Integer> lexicon;
	ArrayList<TreeMap<String,Integer>> sublexica;
	TreeMap <String, ArrayList<Integer>> statistics;
	TreeMap <String, ArrayList<Double>> frequencies;
	TreeMap <String, Double> means;
	TreeMap <String, Double> stdevs;

	int textSize; 
	ArrayList <Integer> chunkSizes;
	int numOfTypes; 
	
	public TrainingLexicon(){
		lexicon = new TreeMap <String, Integer>(); 
		sublexica = new ArrayList<TreeMap<String,Integer> >();
		chunkSizes = new ArrayList <Integer>() ;
	}
	
	public void addChunkOfText(String txt){
		TreeMap <String, Integer> chunklexicon = new TreeMap <String, Integer>(); 
		int ccount=0;
		StringTokenizer wdPrsr = new StringTokenizer(txt, WHITE_SPACE_CHARACTERS); 
		while(wdPrsr.hasMoreTokens()){
			textSize++;
			ccount++;
			String wd = wdPrsr.nextToken();
			Integer lxCnt = lexicon.get(wd);
			if(lxCnt==null){
				lexicon.put(wd, new Integer(1));
			}else{
				lexicon.put(wd, new Integer(lxCnt.intValue()+1));
			}
			Integer slxCnt = chunklexicon.get(wd);
			if(slxCnt==null){
				chunklexicon.put(wd, new Integer(1));
			}else{
				chunklexicon.put(wd, new Integer(lxCnt.intValue()+1));
			}
		}
		sublexica.add(chunklexicon);
		chunkSizes.add(new Integer(ccount));
	}
	public void addText(String txt, int chunkSize){
		TreeMap <String, Integer> chunklexicon = new TreeMap <String, Integer>(); 
		int ccount = 0;
		textSize = 0;
		StringTokenizer wdPrsr = new StringTokenizer(txt, WHITE_SPACE_CHARACTERS); 
		while(wdPrsr.hasMoreTokens()){
			textSize++;
			ccount++;
			String wd = wdPrsr.nextToken();
			Integer lxCnt = lexicon.get(wd);
			if(lxCnt==null){
				lexicon.put(wd, new Integer(1));
			}else{
				lexicon.put(wd, new Integer(lxCnt.intValue()+1));
			}
			Integer slxCnt = chunklexicon.get(wd);
			if(slxCnt==null){
				chunklexicon.put(wd, new Integer(1));
			}else{
				chunklexicon.put(wd, new Integer(slxCnt.intValue()+1));
			}
			if(ccount==chunkSize){
				sublexica.add(chunklexicon);
				chunkSizes.add(new Integer(ccount));
				chunklexicon = new TreeMap <String, Integer>(); 
				ccount=0;
			}
		}
		sublexica.add(chunklexicon);
		chunkSizes.add(new Integer(ccount));
	}
	public void collectStatistics(){
		numOfTypes = lexicon.size();
		int numberOfSublexica = sublexica.size();
		statistics = new TreeMap <String, ArrayList<Integer>>(); 
		List<String> lxKeyList = new  LinkedList<String>(lexicon.keySet());
		Collections.sort(lxKeyList);
		Iterator<String> lxIt = lxKeyList.iterator();
		while(lxIt.hasNext()){
			String key = lxIt.next();
			Integer cnt= lexicon.get(key);
			ArrayList<Integer> stArr = new ArrayList<Integer> (); 
			stArr.add(cnt);
			for(int i=0; i<numberOfSublexica; i++){
				TreeMap <String, Integer> chunklexicon = sublexica.get(i);
				Integer ccnt = chunklexicon.get(key);
				if(ccnt!=null){
					stArr.add(ccnt);
				}else{
					stArr.add(new Integer(0));
				}
			}
			statistics.put(key, stArr);
		}
	}
	public void calculateFrequencies(){
		if(statistics==null){
			this.collectStatistics();
		}
		frequencies = new TreeMap <String, ArrayList<Double>>(); 
		means = new TreeMap <String, Double>();
		stdevs = new TreeMap <String, Double>();

		List<String> stList = new  LinkedList<String>(statistics.keySet());
		Collections.sort(stList);
		Iterator<String> it = stList.iterator();
		while(it.hasNext()){
			String type = it.next();
			ArrayList<Integer> cArr= statistics.get(type);
			int cSz = cArr.size();
			double totalFrq =  cArr.get(0).doubleValue()/(double)textSize*DEFAULT_SCORE_SCALE; 
			means.put(type, new Double(totalFrq));
			ArrayList<Double> fArr = new ArrayList<Double>();
			fArr.add(totalFrq);
			double stdev = 0.0;
			for(int i=1; i<cSz; i++){
				double tmp = totalFrq - cArr.get(i).doubleValue()/chunkSizes.get(i-1).doubleValue()*DEFAULT_SCORE_SCALE;
				stdev += tmp*tmp;
			}
			stdev =stdev/(textSize-1);
			stdev=Math.sqrt(stdev);
			stdevs.put(type, new Double(stdev));

			fArr.add(new Double(stdev));		
			frequencies.put(type, fArr);
		}
	}
	public ArrayList<String> getFequencyList(){
		if(frequencies==null){
			calculateFrequencies();
		}
		TreeMap<Double, ArrayList<String>> frequencyList = new TreeMap<Double, ArrayList<String>>();
		List<String> stList = new  LinkedList<String>(frequencies.keySet());
		Collections.sort(stList);
		Iterator<String> it = stList.iterator();
		while(it.hasNext()){
			String type = it.next();
			ArrayList<Double> fArr= frequencies.get(type);
			Double f = -fArr.get(0);
			if(frequencyList.containsKey(f)){
				ArrayList<String> tmp = frequencyList.get(f); 
				tmp.add(type);
				frequencyList.put(f, tmp);
			}else{
				ArrayList<String> tmp = new ArrayList<String> ();
				tmp.add(type);
				frequencyList.put(f, tmp);
			}
		}
		List<Double> fqList = new  LinkedList<Double>(frequencyList.keySet());
		Collections.sort(fqList);
		Iterator<Double>fqIt = fqList.iterator();
		ArrayList<String> returnList = new ArrayList<String>();
		while(fqIt.hasNext()){
			Double fq = fqIt.next();
			ArrayList<String> tmpList = frequencyList.get(fq);
			if(tmpList!=null){
				int sz = tmpList.size();
				for(int i=0; i<sz; i++){
					String tmpStr = tmpList.get(i);
					returnList.add(tmpStr);
				}
			}
		}
		return returnList;
	}
	public ArrayList<String> getFequencyList(int mfwNum){
		if(frequencies==null){
			calculateFrequencies();
		}
		TreeMap<Double, ArrayList<String>> frequencyList = new TreeMap<Double, ArrayList<String>>();
		List<String> stList = new  LinkedList<String>(frequencies.keySet());
		Collections.sort(stList);
		Iterator<String> it = stList.iterator();
		int cnt = 0; 
		while(it.hasNext()){
			cnt++;
			String type = it.next();
			ArrayList<Double> fArr= frequencies.get(type);
			Double f = -fArr.get(0);
			if(frequencyList.containsKey(f)){
				ArrayList<String> tmp = frequencyList.get(f); 
				tmp.add(type);
				frequencyList.put(f, tmp);
			}else{
				ArrayList<String> tmp = new ArrayList<String> ();
				tmp.add(type);
				frequencyList.put(f, tmp);
			}
		}
		List<Double> fqList = new  LinkedList<Double>(frequencyList.keySet());
		Collections.sort(fqList);
		Iterator<Double>fqIt = fqList.iterator();
		ArrayList<String> returnList = new ArrayList<String>();
		cnt = 0;
		while(fqIt.hasNext()&& cnt < mfwNum){
			Double fq = fqIt.next();
			ArrayList<String> tmpList = frequencyList.get(fq);
			if(tmpList!=null){
				int sz = tmpList.size();
				for(int j=0; j<sz && cnt < mfwNum; j++){
					String tmpStr = tmpList.get(j);
					returnList.add(tmpStr);
					cnt++;
				}
			}
		}
		return returnList;
	}
	public TreeMap<String, ArrayList<Integer>> getStatistics() {
		return statistics;
	}

	public void setStatistics(TreeMap<String, ArrayList<Integer>> statistics) {
		this.statistics = statistics;
	}

	public int getTextSize() {
		return textSize;
	}

	public void setTextSize(int textSize) {
		this.textSize = textSize;
	}

	public ArrayList<Integer> getChunkSizes() {
		return chunkSizes;
	}

	public void setChunkSizes(ArrayList<Integer> chunkSizes) {
		this.chunkSizes = chunkSizes;
	}

	public TreeMap<String, ArrayList<Double>> getFrequencies() {
		return frequencies;
	}

	public void setFrequencies(TreeMap<String, ArrayList<Double>> frequencies) {
		this.frequencies = frequencies;
	}

	public ArrayList<TreeMap<String, Integer>> getSublexica() {
		return sublexica;
	}

	public void setSublexica(ArrayList<TreeMap<String, Integer>> sublexica) {
		this.sublexica = sublexica;
	}

	public TreeMap<String, Double> getStdevs() {
		return stdevs;
	}

	public void setStdevs(TreeMap<String, Double> stdevs) {
		this.stdevs = stdevs;
	}

	public TreeMap<String, Double> getMeans() {
		return means;
	}

	public void setMeans(TreeMap<String, Double> means) {
		this.means = means;
	}
	
}
